package generateVoyages;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import master.DataReader;
import master.Node;

public class OutputWriter {
	
	/**
	 * Takes the voyages generated by the VoyageGenerator, a DataReader instance, and a file name as input. The method writes an input file to be used in Xpress. The file directory should be changed here.
	 * @param voyages
	 * @param input
	 * @param fileName
	 * @throws CloneNotSupportedException
	 */
	public static void writeOutputFile(ArrayList<ArrayList<State>> voyages, DataReader input, String fileName) throws CloneNotSupportedException{
		try {

			//Change the file directory to the wanted location
			File file = new File("//file.stud.iot.ntnu.no/Home/teodorkr/Documents/Master/Mosel/VoyageBased/" + fileName + "_output.txt");
 
			// if file doesnt exists, then create it
			if (!file.exists()) {
				file.createNewFile();
			}
 
			FileWriter fw = new FileWriter(file.getAbsoluteFile());
			BufferedWriter bw = new BufferedWriter(fw);
			
			
			//Writes the node list
			bw.write("Nodes: [");
			for(int i=0; i < input.getNodeList().size(); i++){
				bw.write(input.getNodeList().get(i).getNodeNumber() + " ");
			}
			bw.write("]");
			bw.newLine();
			
			//Writes the pickup node list
			bw.write("PickupNodes: [");
			for(int i=0; i < input.getPickupNodeList().size(); i++){
				bw.write(input.getPickupNodeList().get(i).getNodeNumber() + " ");
			}
			bw.write("]");
			bw.newLine();
			
			//Writes the delivery node list
			bw.write("DeliveryNodes: [");
			for(int i=0; i < input.getDeliveryNodeList().size(); i++){
				bw.write(input.getDeliveryNodeList().get(i).getNodeNumber() + " ");
			}
			bw.write("]");
			bw.newLine();
			
			//Writes the costs associated with sailing a voyage
			bw.write("ServiceSailingCosts: [");
			bw.newLine();
			int voyNum = 0;
			for(int v = 0; v < voyages.size(); v++){
				for(int r = 0; r < voyages.get(v).size(); r++){
					voyNum++;
					bw.write("(" + (v+1) + "," + voyNum +  ")");
					if(v<voyages.size()-1) bw.write(voyages.get(v).get(r).getCost() + " ");
					else{
						bw.write((voyages.get(v).get(r).getCost() + input.getFixedCharterCosts() + 
								Math.ceil(voyages.get(v).get(r).getCurrTime()/24)*input.getTimeCharterCosts()) + " ");
					}
					bw.newLine();
				}
				bw.newLine();
			}
			bw.write("]");
			bw.newLine();
			
			//Writes the number of voyages for each vessel
			bw.write("VoyNum: [");
			for(ArrayList<State> vessel:voyages){
				bw.write(vessel.size() + " ");
			}
			bw.write("]");
			bw.newLine();
			
			//Writes the cost for postponed orders and the cost for delayed OSVs
			bw.write("PostPoneCosts: [");
			double[] priority = input.getPriority();
			for(int i = 0; i<input.getPickupNodeList().size(); i++){
				bw.write((input.getPostponeCosts()*priority[i]) + " ");
			}
			for(int i = 0; i<input.getDeliveryNodeList().size(); i++){
				bw.write((input.getPostponeCosts()*priority[i]) + " ");
			}
			bw.write("]");
			bw.newLine();
			
			bw.newLine();
			bw.write("DelayCosts: " + input.getDelayCosts());
			bw.newLine();
			
			//Writes the matrix A describing all voyages for all vessels
			bw.write("A: [");
			bw.newLine();
			int v = 0;
			int r = 0;
			int n1 = -1;
			int n2 = -1;
			for(ArrayList<State> vessel:voyages){
				v++;
				for(State voyage:vessel){
					ArrayList<Node> path = new ArrayList<Node>();
					State tempState = (State) voyage.clone();
					path.add(tempState.getCurrNode());
					while(tempState.getCurrNode().getNodeNumber() != 0){
						path.add(0, tempState.getPredecessor().getCurrNode());
						tempState = (State) tempState.getPredecessor().clone();
					}
					r++;
					n1 = -1;
					for(Node node:input.getNodeList()){
						n1++;
						n2 = -1;
						for(Node node2:input.getNodeList()){
							n2++;
							if(path.indexOf(node2) != -1 && path.indexOf(node) != -1 && path.indexOf(node2) == path.indexOf(node) + 1){
								bw.write("(" + v + "," + r + "," + n1 + "," + n2 + ")");
								bw.write(1 + " ");
							}
//							else bw.write(0 + " ");
						}
					}
					bw.newLine();
				}
			}
			bw.write("]");
			bw.newLine();
			
			//Writes the time spent for sailing a voyage
			bw.write("ServiceSailingTimes: [");
			bw.newLine();
			v = 0;
			r = 0;
			for(ArrayList<State> vessel:voyages){
				v++;
				for(State voyage:vessel){
					r++;
					bw.write("(" + v + "," + r +  ")");
					bw.write(voyage.getCurrTime() + " ");
				}
				bw.newLine();
			}
			bw.write("]");
			bw.newLine();
			
			//Writes mintime and maxtime
			bw.write("MinTime: [");
			for(int i = 0; i<input.getMinTime().length; i++){
				bw.write(input.getMinTime()[i] + " ");
			}
			bw.write("]");
			bw.newLine();
			
			bw.write("MaxTime: [");
			for(int i = 0; i<input.getMaxTime().length; i++){
				bw.write(input.getMaxTime()[i] + " ");
			}
			bw.write("]");
			bw.newLine();
			
			//Writes the time charter period parameter
			bw.write("H: 24");
			
			bw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
