package generateVoyages;

import master.Node;

public class State implements Cloneable{
	private State predecessor;
	private int[] capacity;
	private double backload;
	private double demand;
	private double cost;
	private double currTime;
	private double[] availableTime;
	private Node currNode;
	private int visited;
	
	/**
	 * Constructor for an initial state
	 * @param currNode
	 * @param capacity
	 * @param cost
	 * @param currTime
	 * @param minTime
	 * @param maxTime
	 */
	public State(Node currNode, int[] capacity, double cost, double currTime, int[] minTime, double[] maxTime){
		this.cost = cost;
		this.currTime = currTime;
		this.capacity = capacity;
		this.backload = 0;
		this.demand = 0;
		
		this.availableTime = new double[capacity.length];
		for(int i=0; i<capacity.length-1; i++){
			this.availableTime[i] = maxTime[i] + 24 - minTime[i]; // Hard time limit 24 hours after maxTime
		}
		this.availableTime[capacity.length-1] = maxTime[capacity.length-1] - minTime[capacity.length-1];
			
		this.currNode = currNode;
		this.visited = (int) Math.pow(2, currNode.getNodeNumber());
	}
	
	/**
	 * Constructor for a state with at least one predecessor
	 * @param predecessor
	 * @param currNode
	 * @param backload
	 * @param demand
	 * @param cost
	 * @param currTime
	 */
	public State(State predecessor, Node currNode, double backload, double demand, double cost, double currTime){
		this.predecessor = predecessor;
		this.capacity = predecessor.getCapacity();
		this.backload = backload;
		this.demand = demand;
		this.cost = cost;
		
		this.currTime = currTime;
		this.availableTime = predecessor.getAvailableTime(); // Hard time limit 24 hours after maxTime
		
		this.currNode = currNode;
		this.visited = predecessor.visited + 2^(currNode.getNodeNumber());
	}
	
	public Object clone() throws CloneNotSupportedException{
		return super.clone();
	}
	
	public Node getCurrNode() {
		return currNode;
	}

	public void setCurrNode(Node currNode) {
		this.currNode = currNode;
	}
	
	public double getCost() {
		return cost;
	}

	public void setCost(double cost) {
		this.cost = cost;
	}

	public double getCurrTime() {
		return currTime;
	}

	public void setCurrTime(double currTime) {
		this.currTime = currTime;
	}

	public double[] getAvailableTime(){
		return availableTime;
	}

	public double getBackload() {
		return backload;
	}

	public void setBackload(double backload) {
		this.backload = backload;
	}

	public double getDemand() {
		return demand;
	}

	public void setDemand(double demand) {
		this.demand = demand;
	}
	
	public int getVisited() {
		return visited;
	}

	public void setVisited(int visited) {
		this.visited = visited;
	}
	
	public void setVisited(Node node){
		this.visited += Math.pow(2, node.getNodeNumber());
	}
	
	public State getPredecessor(){
		return this.predecessor;
	}
	
	public int[] getCapacity(){
		return this.capacity;
	}
}
