package master;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.jxmapviewer.viewer.GeoPosition;

/**
 * Used for reading problem instance files and initialize all nodes and vessels in the instance.
 * @author Nils Albjerk
 * @author Teodor Danielsen
 * @author Stian Krey
 *
 */
public class DataReader {
	int nVessels;
	int[] pickNodes;
	int[] delNodes;
	double[][] serviceSailingCosts;
	double fixedCharterCosts;
	int timeCharterCosts;
	int postponeCosts;
	int delayCosts;
	double[] backload;
	double[] demand;
	int[] capacity;
	double[][] sailingTime;
	double[] maxTime;
	int[] minTime;
	int h;
	double[] latitude;
	double[] longitude;
	String[] name;
	double[] priority;
	
	boolean noInitialFound;
	
	public ArrayList<Vessel> vessels = new ArrayList<Vessel>();
	public ArrayList<Node> nodeList = new ArrayList<Node>();
	public ArrayList<PickupNode> pickupNodeList = new ArrayList<PickupNode>();
	public ArrayList<DeliveryNode> deliveryNodeList = new ArrayList<DeliveryNode>();
	public ArrayList<Node> droppedList = new ArrayList<Node>();
	
	/**
	 * Takes a problem instance file as input, creates all nodes and vessels, and assigns the correct values to their fields
	 * @param fileName the problem instance file to be read
	 */
	public DataReader(String fileName){
		try {
			this.ReadData(fileName);
		} catch (IOException e) {
			e.printStackTrace();
		}
		this.createNodeList();
		this.addNodeCosts();
		this.addSibling();
		this.addVessels();
		this.addPositionAndName();
	}
	
	/**
	 * Reads a problem instance file, and assigns all values to fields in the DataReader class
	 * @param filename the problem instance file to be read
	 * @throws IOException
	 */
	public void ReadData(String filename) throws IOException {
		String nVessels = "";
		String pickNodes = "";
		String delNodes = "";
		String fixedCharterCosts = "";
		String timeCharterCosts = "";
		String postponeCosts = "";
		String delayCosts = "";
		String backload = "";
		String demand = "";
		String capacity = "";
		String maxTime = "";
		String minTime = "";
		String h = "";
		String latitude = "";
		String longitude = "";
		String name = "";
		String priority = "";
		serviceSailingCosts = null;
		sailingTime = null;
		File input = new File(filename);
		BufferedReader br = new BufferedReader(new FileReader(input));
		try{
			String line;
			int arrayCounter = 0;
			
			int row = 0;
			int row1 = 0;
			while((line=br.readLine()) != null){
				int size = 0;
				if(line.isEmpty()){
					arrayCounter++;
				}else{
					if (arrayCounter == 0){
						nVessels = line;
					}
					else if (arrayCounter == 1){
						pickNodes = line;
					}
					else if (arrayCounter == 2){
						delNodes = line;
					}
					else if (arrayCounter == 3){
						if (line.startsWith("Service")){
		
						}
						else{
							String[] vals = line.trim().split("\\s+");
							size = vals.length;
							if(serviceSailingCosts== null){
								serviceSailingCosts = new double[size][size];
							}
							
							for (int col = 0; col < size; col++){
								serviceSailingCosts[row][col] = Double.parseDouble(vals[col]);
							}
							row++;
						}
					}
					else if (arrayCounter == 4){
						fixedCharterCosts = line;
					}
					else if (arrayCounter == 5){
						timeCharterCosts = line;
					}
					else if (arrayCounter == 6){
						postponeCosts = line;
					}
					else if (arrayCounter == 7){
						delayCosts = line;
					}
					else if (arrayCounter == 8){
						backload= line;
					}
					else if (arrayCounter == 9){
						demand = line;
					}
					else if (arrayCounter == 10){
						capacity = line;
					}
					else if (arrayCounter == 11){
						if (line.startsWith("Sailing")){
							
						}else if(line.isEmpty()){
							
						}
						else{
							String[] vals = line.trim().split("\\s+");
							size = vals.length;
							if(sailingTime== null){
								sailingTime = new double[size][size];
							}
							for (int col = 0; col < size; col++){
								sailingTime[row1][col] = Double.parseDouble(vals[col].replaceAll("[^\\d*\\.?\\d*\\s]", ""));
							}
							row1++;
						}
					}
					else if (arrayCounter == 12){
						maxTime = line;
					}
					else if (arrayCounter == 13){
						minTime = line;
					}
					else if (arrayCounter == 14){
						h = line;
					}
					else if (arrayCounter == 15){
						latitude = line;
					}
					else if (arrayCounter == 16){
						longitude = line;
					}
					else if (arrayCounter == 17){
						name = line;
					}
					else if (arrayCounter == 18){
						priority = line;
					}
				}
			}
		}finally{
			br.close();
		}
		this.nVessels = StringToInt(nVessels);
		this.pickNodes = StringToIntArray(pickNodes);
		this.delNodes = StringToIntArray(delNodes);
		this.fixedCharterCosts = StringToDouble(fixedCharterCosts);	
		this.timeCharterCosts = StringToInt(timeCharterCosts);
		this.postponeCosts = StringToInt(postponeCosts);
		this.delayCosts = StringToInt(delayCosts);
		this.backload = StringToDoubleArray(backload);
		this.demand = StringToDoubleArray(demand);
		this.capacity = StringToIntArray(capacity);
		this.maxTime = StringToDoubleArray(maxTime);
		this.minTime = StringToIntArray(minTime);
		this.h = StringToInt(h);
		this.latitude = StringToDoubleArray(latitude);
		this.longitude = StringToDoubleArray(longitude);
		this.name = StringToStringArray(name);
		this.priority = StringToDoubleArray(priority);

		

	}
	
	/**
	 * Takes a String that was read from the input file and parses it to an integer
	 * @param input the input String
	 * @return
	 */
	private int StringToInt (String input){
		input = input.replaceAll("[^0-9]", "").trim();
		int output = Integer.parseInt(input);
		return output;
	}
	
	/**
	 * Takes a String that was read from the input file and parses it to a double
	 * @param input the input String
	 * @return
	 */
	private double StringToDouble (String input){
		input = input.replaceAll("[^\\d*\\.?\\d*]", "").trim();
		double output = Double.parseDouble(input);
		return output;
	}
	
	/**
	 * Takes a String that was read from the input file and parses it to an array of integers
	 * @param input the input String
	 * @return
	 */
	private int[] StringToIntArray (String input){
		input = input.replaceAll("[^0-9\\s]", "").trim();
		input = input.replaceAll("[\\s]", " ");
		String[] temp = input.split(" ");
		int[] output = new int[temp.length];
		for(int i = 0; i < temp.length; i++){
			output[i] = Integer.parseInt(temp[i]);
		}
		
		return output;
	}
	
	/**
	 * Takes a String that was read from the input file and parses it to an array of doubles
	 * @param input the input String
	 * @return
	 */
	private double[] StringToDoubleArray (String input){
		input = input.replaceAll("[^\\d*\\.?\\d*\\s]", "").trim();
		input = input.replaceAll("[\\s]", " ");
		String[] temp = input.split(" ");
		double[] output = new double[temp.length];
		for(int i = 0; i < temp.length; i++){
			output[i] = Double.parseDouble(temp[i]);
		}
		
		return output;
	}
	
	/**
	 * Takes a String that was read from the input file and parses it to an array of Strings
	 * @param input the input String
	 * @return
	 */
	private String[] StringToStringArray (String input){
		String[] temp = input.split(" ");
		String[] output = Arrays.copyOfRange(temp, 1, temp.length);
		return output;
	}
	
	public double getMaxTime(int i) {
		return maxTime[i];
	}
	
	public double[] getMaxTime(){
		return maxTime;
	}

	public void setMaxTime(double[] maxTime) {
		this.maxTime = maxTime;
	}

	public int getMinTime(int i) {
		return minTime[i];
	}
	
	public int[] getMinTime(){
		return minTime;
	}

	public void setMinTime(int[] minTime) {
		this.minTime = minTime;
	}
	
	/**
	 * Creates the pickup node list, the delivery node list, and the list containing all nodes
	 */
	public void createNodeList(){
		for (int i = 0; i < pickNodes.length; i++) {
			PickupNode pickupNode = new PickupNode(backload[i], pickNodes[i], priority[i]);
			pickupNodeList.add(pickupNode);
			nodeList.add(pickupNode);
		}
		for (int i = 0; i < delNodes.length; i++) {
			DeliveryNode deliveryNode = new DeliveryNode(demand[i], delNodes[i], priority[i]);
			deliveryNodeList.add(deliveryNode);
			nodeList.add(deliveryNode);
		}
	}
	

	/**
	 * Initializes all the vessels in the problem with their field values
	 */
	public void addVessels(){
		for (int i = 0; i < nVessels; i++) {
			if(i == nVessels-1){ //Make spot vessel
				Vessel vessel = new Vessel(getCapacity(i),getMaxTime(i), getMinTime(i), fixedCharterCosts, timeCharterCosts, i, postponeCosts, delayCosts);
				vessel.spot = true;
				vessels.add(vessel);
			}
			else{
				Vessel vessel = new Vessel(getCapacity(i),getMaxTime(i), getMinTime(i), i, postponeCosts, delayCosts);
				vessel.spot = false;
				vessels.add(vessel);
			}
			
			
			
		}
	}
	
	/**
	 * Adds the service sailing costs and service sailing times for sailing from one node to all other nodes. 
	 * Takes into account if there is different number of pickup and delivery nodes
	 */
	public void addNodeCosts(){
		int pickSize = pickupNodeList.size();
		int delSize = deliveryNodeList.size();
		//If we have the same amount of nodes 
		if(pickSize == delSize){
			for (int i = 0; i < pickSize; i++) {
				PickupNode from = pickupNodeList.get(i);
				for (int j = 0; j < pickSize; j++) {
					PickupNode to = pickupNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j]);
				}
				for (int j = 0; j < delSize; j++) {
					DeliveryNode to = deliveryNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j + delSize]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j + delSize]);
				}
			}
			
			for (int i = delSize; i < 2*delSize; i++) {
				DeliveryNode from = deliveryNodeList.get(i - delSize);
				for (int j = 0; j < pickSize; j++) {
					PickupNode to = pickupNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j]);
				}
				for (int j = 0; j < delSize; j++) {
					DeliveryNode to = deliveryNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j + delSize]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j + delSize]);
				}
			}
		//More pickup than delivery
		}else if(pickSize>delSize){
			for (int i = 0; i < pickSize; i++) {
				PickupNode from = pickupNodeList.get(i);
				for (int j = 0; j < pickSize; j++) {
					PickupNode to = pickupNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j]);
				}
				for (int j = 0; j < delSize; j++) {
					DeliveryNode to = deliveryNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j + pickSize]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j + pickSize]);
				}
			}
			
			for (int i = pickSize; i < (delSize+pickSize); i++) {
				DeliveryNode from = deliveryNodeList.get(i - pickSize);
				for (int j = 0; j < pickSize; j++) {
					PickupNode to = pickupNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j]);
				}
				for (int j = 0; j < delSize; j++) {
					DeliveryNode to = deliveryNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j + pickSize]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j + pickSize]);
				}
			}
		//More delivery than pickup
		}else if(pickSize<delSize){
			for (int i = 0; i < pickSize; i++) {
				PickupNode from = pickupNodeList.get(i);
				for (int j = 0; j < pickSize; j++) {
					PickupNode to = pickupNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j]);
				}
				for (int j = 0; j < delSize; j++) {
					DeliveryNode to = deliveryNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j + delSize-1]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j + delSize-1]);
				}
			}
			
			for (int i = (delSize-1); i < (delSize+pickSize); i++) {
				DeliveryNode from = deliveryNodeList.get(i - (delSize-1));
				for (int j = 0; j < pickSize; j++) {
					PickupNode to = pickupNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j]);
				}
				for (int j = 0; j < delSize; j++) {
					DeliveryNode to = deliveryNodeList.get(j);
					from.addSailingTime(to, sailingTime[i][j + delSize-1]);
					from.addServiceSailingCosts(to, serviceSailingCosts[i][j + delSize-1]);
				}
			}
		}else{
			//Do nothing
		}

	}
	
	/**
	 * Sets the sibling field of a node, if it has a sibling in the problem
	 */
	public void addSibling(){
		int pickSize = pickupNodeList.size();
		int delSize = deliveryNodeList.size();
		if(pickSize==delSize){
			for (int i = 0; i < pickSize; i++) {
				pickupNodeList.get(i).setSibling(deliveryNodeList.get(i));
				deliveryNodeList.get(i).setSibling(pickupNodeList.get(i));
			}
		}else if(pickSize>delSize){
			for (int i = 0; i < delSize; i++) {
				pickupNodeList.get(i).setSibling(deliveryNodeList.get(i));
				deliveryNodeList.get(i).setSibling(pickupNodeList.get(i));
			}
		}else if(pickSize<delSize){
			for (int i = 0; i < pickSize; i++) {
				pickupNodeList.get(i).setSibling(deliveryNodeList.get(i));
				deliveryNodeList.get(i).setSibling(pickupNodeList.get(i));
			}
		}else{
			//Do nothing
		}
	}
	
	/**
	 * Adds the GeoPositions and abbreviated names of all nodes, used to draw a map of the final solution's routes
	 */
	public void addPositionAndName(){
		for (int i = 0; i < pickupNodeList.size(); i++) {
			GeoPosition position = new GeoPosition(latitude[i], longitude[i]);
			pickupNodeList.get(i).setPosition(position);
			pickupNodeList.get(i).setNodeName(name[i]+"(p)");
		}
		for (int i = 0; i < deliveryNodeList.size(); i++) {
			GeoPosition position = new GeoPosition(latitude[i], longitude[i]);
			deliveryNodeList.get(i).setPosition(position);
			deliveryNodeList.get(i).setNodeName(name[i]+"(d)");
		}
	}

	
	public int getnVessels() {
		return nVessels;
	}
		
	public List<Vessel> getVessels(){
		return this.vessels;
	}
	
	public Node getNode (int i){
		return nodeList.get(i);
	}
	
	public ArrayList<Node> getNodeList(){
		return this.nodeList;
	}
	
	public ArrayList<PickupNode> getPickupNodeList(){
		return this.pickupNodeList;
	}
	
	public ArrayList<DeliveryNode> getDeliveryNodeList(){
		return this.deliveryNodeList;
	}
	
	public int getCapacity(int i){
		return capacity[i];
	}
	
	public double[] getPriority() {
		return priority;
	}

	public int[] getCapacity(){
		return capacity;
	}

	public double getFixedCharterCosts() {
		return fixedCharterCosts;
	}

	public int getTimeCharterCosts() {
		return timeCharterCosts;
	}

	public int getPostponeCosts() {
		return postponeCosts;
	}

	public int getDelayCosts() {
		return delayCosts;
	}
	
	
	public double TotalCost(){
		double totalCost = 0;
	
		for (int i = 0; i < this.vessels.size(); i++) {
			totalCost += this.vessels.get(i).cost();
		}
		return Math.round(totalCost);
	}
	
	public void printResult(){
		for (int i = 0; i < vessels.size(); i++) {
			vessels.get(i).printVoyage();
			System.out.println("MinTime:			"+ vessels.get(i).minTime);
			System.out.println("Voyagae duration:		"+ (vessels.get(i).voyageSailingTime()));
			System.out.println("MaxTime:			"+ vessels.get(i).maxTime);
			System.out.println("Ship delayed:	"+ (vessels.get(i).voyageSailingTime() + vessels.get(i).minTime > vessels.get(i).maxTime ));
			System.out.println("Capacity ok?:	"+ vessels.get(i).voyageCapOK());
			System.out.println("Spot:		"+ vessels.get(i).spot);
			System.out.println("Voyage cost:		" + vessels.get(i).cost());
			System.out.println("-----------------------------");	
			}
		System.out.println("Total cost: "+ TotalCost());
		}
		

}
