package master;


import java.util.ArrayList;
import java.util.Random;

import master.DataReader;
import master.DeliveryNode;
import master.PickupNode;

/**
 * An insertion algorithm based on randomized insertions, limited by the capacity and time limit constraints of the problem instances
 * @author Nils Albjerk
 * @author Teodor Danielsen
 * @author Stian Krey
 *
 */
public class InitialSolution {
	private long seed = 42;
	private long seed2 = 42;
	private Random rn = new Random(seed);
	private Random r = new Random(seed2);
	
	/**
	 * Creates imax initial solutions and selects the solution with the lowest total cost as the best solution
	 * @param data a DataReader object that contains the problem instance
	 * @param imax the number of initial solutions to be constructed
	 * @return the solution found with lowest total cost
	 */
	public Solution createInitialVoyages(DataReader data, int imax){
		int iterations = imax;
		int dropCounter = 1;
		ArrayList<Solution> solutions = new ArrayList<Solution>();
		
		for (long k = 0; k < iterations/2; k++) {
			ArrayList<Vessel> fleetCopy = vesselsCloning(data.vessels);
			
			Solution sol = new Solution(fleetCopy, data.postponeCosts);
			solutions.add(sol);

		
			ArrayList<PickupNode> tempPick = new ArrayList<PickupNode>();
			ArrayList<DeliveryNode> tempDel= new ArrayList<DeliveryNode>();
			
			for (int i = 0; i < data.pickupNodeList.size(); i++) {
				tempPick.add(data.pickupNodeList.get(i));
			}
			for (int i = 0; i < data.deliveryNodeList.size(); i++) {
				tempDel.add(data.deliveryNodeList.get(i));
			}

			
			for (int i = 0; i <sol.fleet.size() ; i++) {	
				
				sol.fleet.get(i).route.add(tempPick.get(0));	//add start depot
				sol.fleet.get(i).route.add(tempDel.get(0));	//add end depot
			
			}
			tempPick.remove(0);
			tempDel.remove(0);
			
			if (dropCounter<tempDel.size()) {
				sol.removedNodes.add(tempDel.get(dropCounter));
				if(tempDel.get(dropCounter).getSibling() != null){
					sol.removedNodes.add(tempDel.get(dropCounter).sibling);
					tempPick.remove(tempDel.get(dropCounter).sibling);
				}
				
				tempDel.remove(dropCounter);
			}
			
			int delCounter = 0;
			int pickCounter = 0;
		
			while (!tempDel.isEmpty() || !tempPick.isEmpty()) {
				int rand = rn.nextInt(sol.fleet.size()) ; 
				
				if (delCounter > 10) {
					while(!tempDel.isEmpty()) {
						sol.removedNodes.add(tempDel.get(0));
						tempDel.remove(0);
					}
				}
				
				if (pickCounter > 10) {
					while (!tempPick.isEmpty()) {
						sol.removedNodes.add(tempPick.get(0));
						tempPick.remove(0);
					}
				}
				
				if (!tempDel.isEmpty()) {
					int randDel = r.nextInt(tempDel.size());
					
					sol.fleet.get(rand).route.add(sol.fleet.get(rand).route.size()-1, tempDel.get(randDel));
					
					if (sol.allVesselsOK()) {
						if (tempDel.get(randDel).getSibling() != null) {
							if (tempPick.contains(tempDel.get(randDel).getSibling())) {
								sol.fleet.get(rand).route.add(sol.fleet.get(rand).route.size()-1,tempDel.get(randDel).getSibling());
								if (sol.allVesselsOK()) {
									tempPick.remove(tempDel.get(randDel).getSibling());
								}
								else {
									sol.fleet.get(rand).route.remove(tempDel.get(randDel).getSibling());
									pickCounter++;
								}
								
							}
						}
						tempDel.remove(tempDel.get(randDel));
					}
					else {
						sol.fleet.get(rand).route.remove(tempDel.get(randDel));
						delCounter++;
					}
					
				}
				else if(!tempPick.isEmpty()){
					int randPick = r.nextInt(tempPick.size());
					
					sol.fleet.get(rand).route.add(sol.fleet.get(rand).route.size()-1, tempPick.get(randPick));
					if (sol.allVesselsOK()) {
						tempPick.remove(randPick);
					}
					else {
						sol.fleet.get(rand).route.remove(tempPick.get(randPick));
						pickCounter++;
					}
					
				}
			}
			dropCounter++;
		}
		
//		----------------
		dropCounter=1;
		for (int k = 0; k < iterations/2; k++) {
			ArrayList<Vessel> fleetCopy = vesselsCloning(data.vessels);
			
			Solution sol = new Solution(fleetCopy, data.postponeCosts);
			solutions.add(sol);

		
			ArrayList<PickupNode> tempPick = new ArrayList<PickupNode>();
			ArrayList<DeliveryNode> tempDel= new ArrayList<DeliveryNode>();
			
			for (int i = 0; i < data.pickupNodeList.size(); i++) {
				tempPick.add(data.pickupNodeList.get(i));
			}
			for (int i = 0; i < data.deliveryNodeList.size(); i++) {
				tempDel.add(data.deliveryNodeList.get(i));
			}

			
			for (int i = 0; i <sol.fleet.size() ; i++) {	
				
				sol.fleet.get(i).route.add(tempPick.get(0));	//add start depot
				sol.fleet.get(i).route.add(tempDel.get(0));	//add end depot
			
			}
			tempPick.remove(0);
			tempDel.remove(0);
			
			if (dropCounter<tempDel.size()) {
				sol.removedNodes.add(tempDel.get(dropCounter));
				if(tempDel.get(dropCounter).getSibling() != null){
					sol.removedNodes.add(tempDel.get(dropCounter).sibling);
					tempPick.remove(tempDel.get(dropCounter).sibling);
				}
				tempDel.remove(dropCounter);
			}
			
			int delCounter = 0;
			int pickCounter = 0;
		
			while (!tempDel.isEmpty() || !tempPick.isEmpty()) {
				int rand = rn.nextInt(sol.fleet.size()-1) ; 
				
				if (delCounter > 10) {
					while(!tempDel.isEmpty()) {
						sol.removedNodes.add(tempDel.get(0));
						tempDel.remove(0);
					}
				}
				
				if (pickCounter > 10) {
					while (!tempPick.isEmpty()) {
						sol.removedNodes.add(tempPick.get(0));
						tempPick.remove(0);
					}
				}
				
				if (!tempDel.isEmpty()) {
					int randDel = r.nextInt(tempDel.size());
					
					sol.fleet.get(rand).route.add(sol.fleet.get(rand).route.size()-1, tempDel.get(randDel));
					
					if (sol.allVesselsOK()) {
						if (tempDel.get(randDel).getSibling() != null) {
							if (tempPick.contains(tempDel.get(randDel).getSibling())) {
								sol.fleet.get(rand).route.add(sol.fleet.get(rand).route.size()-1,tempDel.get(randDel).getSibling());
								if (sol.allVesselsOK()) {
									tempPick.remove(tempDel.get(randDel).getSibling());
								}
								else {
									sol.fleet.get(rand).route.remove(tempDel.get(randDel).getSibling());
									pickCounter++;
								}
								
							}
						}
						tempDel.remove(tempDel.get(randDel));
					}
					else {
						sol.fleet.get(rand).route.remove(tempDel.get(randDel));
						delCounter++;
					}
					
				}
				else if(!tempPick.isEmpty()){
					int randPick = r.nextInt(tempPick.size());
					
					sol.fleet.get(rand).route.add(sol.fleet.get(rand).route.size()-1, tempPick.get(randPick));
					if (sol.allVesselsOK()) {
						tempPick.remove(randPick);
					}
					else {
						sol.fleet.get(rand).route.remove(tempPick.get(randPick));
						pickCounter++;
					}
					
				}
			}
			dropCounter++;
		}
//		--------------------------
		
		int winnerIndex = 0;
		double winnerCost = 99999999;
		
		for (int i = 0; i < solutions.size(); i++) {
			if (solutions.get(i).TotalCostInternal() <winnerCost) {
				winnerIndex = i;
				winnerCost = solutions.get(i).TotalCostInternal();
			}
		}
		solutions.get(winnerIndex).initNum=winnerIndex;
		
		if (winnerCost == 99999999) {
			System.out.println("No initial found");
			return null;
		}
		else{
			System.out.println("best initial is solution #"+ winnerIndex +" and has cost: "+ Math.round(solutions.get(winnerIndex).TotalCostInternal()*100)/100d + " and droppped nodes: "+ !solutions.get(winnerIndex).removedNodes.isEmpty());
			return solutions.get(winnerIndex);
		}
	}
	
	
	/**
	 * Creates a deep clone of a list of vessels
	 * @param vessels the list of vessels to be cloned
	 * @return the deep cloned list
	 */
	public ArrayList<Vessel> vesselsCloning(ArrayList<Vessel> vessels){
		ArrayList<Vessel> vesselsClone = new ArrayList<Vessel>();
		
		for (int i = 0; i < vessels.size(); i++) {
			Vessel tempVes = new Vessel(vessels.get(i));
			vesselsClone.add(tempVes);
		}
		return vesselsClone;
	}
	
	
	
}


	
