package master;




import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;


public class Main {
	/**
	 * The main class for running the VNS heuristic. In the data provided, the vessels of several instances all have 
	 * minTime set to 0, implying that they all start from the depot at the same time. Increasing the minTime and maxTime
	 * by the same amount of time for some of the vessels, should not alter the solutions obtained, since none of the nodes
	 * visited have time windows. 
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		String outputFileLocation = "//file.stud.iot.ntnu.no/Home/stiankre/Documents/01juni";
		/*
		 *  These loops make it possible to automatically run all test instances ten times
		 *  with their correct parameters and print the results to files at the location specified above
		 *  s-loop : the problem files to be run
		 *  q-loop : the number of times each instance is run
		 */
		for (int s = 0; s < 31; s++) {
			String file = "";
			int inits 	= 0;
			double nMax = 0;
			double pMax = 0;
			int sMax 	= 0;
			int size 	= 0;
			HashMap<Integer, Integer> initH 	= new HashMap<Integer, Integer>();
			HashMap<Integer, Double>  nMaxH 	= new HashMap<Integer, Double>();
			HashMap<Integer, Double> pMaxH 		= new HashMap<Integer, Double>();
			HashMap<Integer, Integer> sMaxH 	= new HashMap<Integer, Integer>();
			initH.put(1, 5000); 	initH.put(2, 50000); 	initH.put(3, 100000); 
			nMaxH.put(1, 0.1); 		nMaxH.put(2, 0.7); 		nMaxH.put(3, 0.5); 
			pMaxH.put(1, 0.5); 		pMaxH.put(2, 2.0); 		pMaxH.put(3, 2.0); 
			sMaxH.put(1, 5); 		sMaxH.put(2, 20); 		sMaxH.put(3, 20); 

			for (int q = 0; q < 10; q++) {
				
				switch (s) {
					case 0:
						size = 1;
						file = "Mongstad_4";
						
						break;
					case 1:
						size = 1;
						file = "Mongstad_4_Speed";
						break;
					case 2:
						size = 1;
						file = "Mongstad_4_Triple";
						break;
					case 3:
						size = 1;
						file = "Mongstad_5";
						break;
					case 4:
						size = 1;
						file = "Mongstad_5_Speed";
						break;
					case 5:
						size = 1;
						file = "Mongstad_5_Triple";
						break;
					case 6:
						size = 1;
						file = "Agotnes_6";
						break;
					case 7:
						size = 1;
						file = "Agotnes_6_Speed";
						break;
					case 8:
						size = 1;
						file = "Agotnes_6_Triple";
						break;
					case 9:
						size = 1;
						file = "Floro_7";
						break;
					case 10:
						size = 1;
						file = "Floro_7_Speed";
						break;
					case 11:
						size = 1;
						file = "Floro_7_Triple";
						break;
					case 12:
						size = 2;
						file = "Agotnes_8";
						break;
					case 13:
						size = 2;
						file = "Agotnes_8_Speed";
						break;
					case 14:
						size = 2;
						file = "Agotnes_8_Triple";
						break;
					case 15:
						size = 2;
						file = "Agotnes_8_1ExtraDel";
					case 16:
						size = 2;
						file = "Mongstad_10";
						break;
					case 17:
						size = 2;
						file = "Mongstad_10_Speed";
						break;
					case 18:
						size = 2;
						file = "Mongstad_10_Triple";
						break;
					case 19:
						size = 2;
						file = "Floro_11";
						break;
					case 20:
						size = 2;
						file = "Floro_11_Speed";
						break;
					case 21:
						size = 2;
						file = "Floro_11_Triple";
						break;
					case 22:
						size = 2;
						file = "Agotnes_12";
						break;
					case 23:
						size = 2;
						file = "Agotnes_12_Speed";
						break;
					case 24:
						size = 2;
						file = "Agotnes_12_Triple";
						break;
					case 25:
						size = 3;
						file = "Mongstad_20";
						break;
					case 26:
						size = 3;
						file = "Mongstad_20_Speed";
						break;
					case 27:
						size = 3;
						file = "Mongstad_20_Triple";
						break;
					case 28:
						size = 3;
						file = "Mongstad_26";
						break;
					case 29:
						size = 3;
						file = "Mongstad_26_Speed";
						break;
					case 30:
						size = 3;
						file = "Mongstad_26_Triple";
						break;
					default:
						break;
				}
				inits = initH.get(size);
				nMax = nMaxH.get(size);
				pMax = pMaxH.get(size);
				sMax = sMaxH.get(size);
				long startTime = System.nanoTime();
				System.out.println("Starting main");

				System.out.println("Start dataReader");
				DataReader data = new DataReader(file+".txt");		
				System.out.println("Finished dataReader");
				
				long initStart = System.nanoTime();
				InitialSolution insert = new InitialSolution();
				Solution solution = insert.createInitialVoyages(data, inits);
				long initEnd = System.nanoTime();
				System.out.println("Inittime: "+ (initEnd-initStart)/1000000 + " ms");
				if (data.noInitialFound) {
		
					return;
				}
				else{					
					System.out.println("Initial solution:");
					solution.printResult();
					System.out.println("---------------------------------------------------------------------------------");
					System.out.println("-----------------------------Starting VNS----------------------------------------");
					VNS VNSAlgorithm = new VNS();
					solution = VNSAlgorithm.VNSAlgortihm(solution, nMax, pMax, sMax, file, outputFileLocation);
					System.out.println("-----------------------------VNS finished----------------------------------------");
					
					System.out.println("----------------------------Final results:-----------------------------------------");
					
					
					/*
					 * Print map for the solution
					 */
//					MapPainter mp = new MapPainter();
//					mp.paintMap(solution);

					/*
					 * Printing to console
					 */
					solution.printResult();
					long endTime = System.nanoTime();
					System.out.println("Runtime "+ (endTime-startTime)/1000000 + " ms");
					System.out.println("");
					solution.routeLoadPrint();
					System.out.println("");
					System.out.println("Finished");					
					
					

			
					/*
					 * Printing to files
					 */
					//File for all results:
					String allFile = outputFileLocation+"/"+file+"_n_"+nMax+"_p_"+pMax+"_s_"+sMax+"_inits_"+inits+"_all.txt";
					String otFile = outputFileLocation+"/"+file+"_n_"+nMax+"_p_"+pMax+"_s_"+sMax+"_inits_"+inits+"_ot.txt";
					
					File file_all = new File(allFile);
					//File for objective and time:
					File file_ot = new File(otFile);
			 
					// if file doesnt exists, then create it
					if (!file_all.exists()) {
						file_all.createNewFile();
					}
					if (!file_ot.exists()) {
						file_ot.createNewFile();
					}
			
					FileWriter fw_all = new FileWriter(file_all.getAbsoluteFile(), true);
					BufferedWriter bw_all = new BufferedWriter(fw_all);
					
					FileWriter fw_ot = new FileWriter(file_ot.getAbsoluteFile(), true);
					BufferedWriter bw_ot = new BufferedWriter(fw_ot);
					
					
					//Only objective and time:
					bw_ot.write( Math.round(solution.TotalCostInternal()) + "	"+(endTime-startTime)/1000000);
					bw_ot.newLine();
					bw_ot.close();
					
					
					//All results:
					bw_all.write("###################################################################################");
					bw_all.newLine();
					bw_all.newLine();
					for (int i = 0; i < solution.fleet.size(); i++) {
						bw_all.write("Vessel #: "+ (solution.fleet.get(i).vesselName));
						bw_all.newLine();
						bw_all.write("Route: ");
						for (int j = 0; j < solution.fleet.get(i).route.size(); j++) {
							bw_all.write((solution.fleet.get(i).route.get(j).getNodeNumber()) +", ");
						}
						bw_all.newLine();
						bw_all.write("MinTime:			"+ solution.fleet.get(i).minTime);
						bw_all.newLine();
						bw_all.write("Voyagae duration:		"+ (Math.round(solution.fleet.get(i).voyageSailingTime()*100)/100d));
						bw_all.newLine();
						bw_all.write("MaxTime:			"+ solution.fleet.get(i).maxTime);
						bw_all.newLine();
						bw_all.write("Ship delayed with:		"+ Math.round((solution.fleet.get(i).voyageSailingTime() + solution.fleet.get(i).minTime  - solution.fleet.get(i).maxTime )*100)/100d);
						bw_all.newLine();
						double startLoad = 0;
						for (int j = 0; j < solution.fleet.get(i).route.size(); j++) {
							if(solution.fleet.get(i).route.get(j) instanceof DeliveryNode){
								DeliveryNode d = (DeliveryNode) solution.fleet.get(i).route.get(j);
								startLoad = startLoad + d.getDemand(); 
							
							}	
						}
						bw_all.write("Start load:			"+ startLoad);
						bw_all.newLine();
						bw_all.write("Capacity ok:			"+ solution.fleet.get(i).voyageCapOK());
						bw_all.newLine();
						bw_all.write("Spot:				"+ solution.fleet.get(i).spot);
						bw_all.newLine();
						bw_all.write("Voyage cost:			" + Math.round(solution.fleet.get(i).cost()*100)/100d);
						bw_all.newLine();
						bw_all.write("-------------------------------------");	
						bw_all.newLine();
					}
					bw_all.write("Nodes removed ");
					for (int i = 0; i < solution.removedNodes.size(); i++) {
						bw_all.write(solution.removedNodes.get(i).getNodeNumber()+", ");
					}
					bw_all.newLine();
					bw_all.write("Total cost: "+ Math.round(solution.TotalCostInternal()));
					bw_all.newLine();
					bw_all.newLine();
					
					bw_all.close();
					
				}
		}
	}

	}
}

