package master;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JToolTip;
import javax.swing.event.MouseInputListener;

import org.jxmapviewer.JXMapKit;
import org.jxmapviewer.JXMapViewer;
import org.jxmapviewer.OSMTileFactoryInfo;
import org.jxmapviewer.input.CenterMapListener;
import org.jxmapviewer.input.PanKeyListener;
import org.jxmapviewer.input.PanMouseInputListener;
import org.jxmapviewer.input.ZoomMouseWheelListenerCursor;
import org.jxmapviewer.painter.CompoundPainter;
import org.jxmapviewer.painter.Painter;
import org.jxmapviewer.viewer.DefaultTileFactory;
import org.jxmapviewer.viewer.DefaultWaypoint;
import org.jxmapviewer.viewer.GeoPosition;
import org.jxmapviewer.viewer.TileFactoryInfo;
import org.jxmapviewer.viewer.Waypoint;
import org.jxmapviewer.viewer.WaypointPainter;

/**
 * Creates maps with routes
 * @author Nils Albjerk
 * @author Teodor Danielsen
 * @author Stian Krey
 *
 */
public class MapPainter {
	/**
	 * Creates a map with routes based on the solution provided
	 * @param solution the solution to be illustrated with a map
	 */
	public void paintMap(Solution solution){
		final JXMapViewer mapViewer = new JXMapViewer();
		
		
				
				// Create a TileFactoryInfo for OpenStreetMap
				TileFactoryInfo info = new OSMTileFactoryInfo();
				DefaultTileFactory tileFactory = new DefaultTileFactory(info);
				tileFactory.setThreadPoolSize(8);
				mapViewer.setTileFactory(tileFactory);
				
				// Add interactions
				MouseInputListener mia = new PanMouseInputListener(mapViewer);
				mapViewer.addMouseListener(mia);
				mapViewer.addMouseMotionListener(mia);

				mapViewer.addMouseListener(new CenterMapListener(mapViewer));
				
				mapViewer.addMouseWheelListener(new ZoomMouseWheelListenerCursor(mapViewer));
				
				mapViewer.addKeyListener(new PanKeyListener(mapViewer));
				
				// Add a selection painter
				SelectionAdapter sa = new SelectionAdapter(mapViewer); 
				SelectionPainter sp = new SelectionPainter(sa); 
				mapViewer.addMouseListener(sa); 
				mapViewer.addMouseMotionListener(sa); 
				mapViewer.setOverlayPainter(sp);
				
				// Create tracks from the geo-positions of the voyages
				List<GeoPosition> all = new ArrayList<GeoPosition>();
				List<GeoPosition> track1 = new ArrayList<GeoPosition>();
				List<GeoPosition> track2 = new ArrayList<GeoPosition>();
				List<GeoPosition> track3 = new ArrayList<GeoPosition>();
				List<GeoPosition> track4 = new ArrayList<GeoPosition>();
				List<GeoPosition> track5 = new ArrayList<GeoPosition>();
				List<GeoPosition> track6 = new ArrayList<GeoPosition>();
				
				all.add(solution.fleet.get(0).route.get(0).getPosition());
				for (int i = 0; i < solution.fleet.size(); i++) {
					for (int j = 1; j < solution.fleet.get(i).route.size()-1; j++) {
						all.add(solution.fleet.get(i).route.get(j).getPosition());
					}
				}
				// Create a compound painter that uses both the route-painter and the waypoint-painter
				List<Painter<JXMapViewer>> painters = new ArrayList<Painter<JXMapViewer>>();
				
				// Add all the routes to the painters
				for (int i = 0; i < solution.fleet.size(); i++) {
					switch (i) {
					case 0:
						for (int j = 0; j < solution.fleet.get(i).route.size(); j++) {
							track1.add(solution.fleet.get(i).route.get(j).getPosition());
						}
						RoutePainter routePainter1 = new RoutePainter(track1, Color.BLUE,0.3f);
						painters.add(routePainter1);
						break;
						
					case 1:
						for (int j = 0; j < solution.fleet.get(i).route.size(); j++) {
							track2.add(solution.fleet.get(i).route.get(j).getPosition());
						}
						
						RoutePainter routePainter2 = new RoutePainter(track2, Color.RED,0.3f);
						painters.add(routePainter2);
						break;
					case 2:
						for (int j = 0; j < solution.fleet.get(i).route.size(); j++) {
							track3.add(solution.fleet.get(i).route.get(j).getPosition());
						}
						
						RoutePainter routePainter3 = new RoutePainter(track3, Color.GREEN,0.3f);
						painters.add(routePainter3);
						break;
					case 3:
						for (int j = 0; j < solution.fleet.get(i).route.size(); j++) {
							track4.add(solution.fleet.get(i).route.get(j).getPosition());
						}
						
						RoutePainter routePainter4 = new RoutePainter(track4, Color.YELLOW,0.3f);
						painters.add(routePainter4);
						break;
					case 4:
						for (int j = 0; j < solution.fleet.get(i).route.size(); j++) {
							track5.add(solution.fleet.get(i).route.get(j).getPosition());
						}
						
						RoutePainter routePainter5 = new RoutePainter(track5, Color.CYAN,0.3f);
						painters.add(routePainter5);
						break;
					case 5:
						for (int j = 0; j < solution.fleet.get(i).route.size(); j++) {
							track6.add(solution.fleet.get(i).route.get(j).getPosition());
						}
						
						RoutePainter routePainter6 = new RoutePainter(track6, Color.BLACK,0.3f);
						painters.add(routePainter6);
						break;
					default:
						break;
					}
				}
				
				
				// Set the focus
//				mapViewer.zoomToBestFit(new HashSet<GeoPosition>(all), 0.7);
				mapViewer.setZoom(12);
				mapViewer.setAddressLocation(solution.fleet.get(0).route.get(0).getPosition());
				
				// Create waypoints from the geo-positions
				Set<MyWaypoint> waypoints = new HashSet<MyWaypoint>();
				for (int i = 0; i < solution.fleet.size(); i++) {
					for (int j = 0; j < solution.fleet.get(i).route.size(); j++) {
						waypoints.add(new MyWaypoint(solution.fleet.get(i).route.get(j).getNodeName().replaceAll("[^A-Z]", ""),Color.LIGHT_GRAY,solution.fleet.get(i).route.get(j).getPosition()));
					}
				}
				// Create a waypoint painter that takes all the waypoints
				WaypointPainter<MyWaypoint> waypointPainter = new WaypointPainter<MyWaypoint>();
				waypointPainter.setWaypoints(waypoints);
				waypointPainter.setRenderer(new FancyWaypointRenderer());
				
				// Add the waypoints to the painters
				painters.add(waypointPainter);
				
				CompoundPainter<JXMapViewer> painter = new CompoundPainter<JXMapViewer>(painters);
				mapViewer.setOverlayPainter(painter);
				
				// Display the viewer in a JFrame
				final JFrame frame = new JFrame(solution.fleet.get(0).route.get(0).getNodeName());
				frame.add(new JLabel("Use left mouse button to pan and mouse wheel to zoom"), BorderLayout.NORTH);
				frame.getContentPane().add(mapViewer);
				frame.setSize(800, 600);
				frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				frame.setVisible(true);
				
				mapViewer.addPropertyChangeListener("zoom", new PropertyChangeListener()
				{
					@Override
					public void propertyChange(PropertyChangeEvent evt)
					{
						updateWindowTitle(frame, mapViewer);
					}
				});
				
				mapViewer.addPropertyChangeListener("center", new PropertyChangeListener()
				{
					@Override
					public void propertyChange(PropertyChangeEvent evt)
					{
						updateWindowTitle(frame, mapViewer);
					}
				});
				
				updateWindowTitle(frame, mapViewer);
	}
	
	
	protected static void updateWindowTitle(JFrame frame, JXMapViewer mapViewer)
	{
		double lat = mapViewer.getCenterPosition().getLatitude();
		double lon = mapViewer.getCenterPosition().getLongitude();
		int zoom = mapViewer.getZoom();
		
		frame.setTitle(String.format("Vessel schedule (%.2f / %.2f) - Zoom: %d", lat, lon, zoom)); 
	}
}
