package master;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JToolTip;
import javax.swing.event.MouseInputListener;

import org.jxmapviewer.JXMapKit;
import org.jxmapviewer.JXMapViewer;
import org.jxmapviewer.OSMTileFactoryInfo;
import org.jxmapviewer.input.CenterMapListener;
import org.jxmapviewer.input.PanKeyListener;
import org.jxmapviewer.input.PanMouseInputListener;
import org.jxmapviewer.input.ZoomMouseWheelListenerCursor;
import org.jxmapviewer.painter.CompoundPainter;
import org.jxmapviewer.painter.Painter;
import org.jxmapviewer.viewer.DefaultTileFactory;
import org.jxmapviewer.viewer.DefaultWaypoint;
import org.jxmapviewer.viewer.GeoPosition;
import org.jxmapviewer.viewer.TileFactoryInfo;
import org.jxmapviewer.viewer.Waypoint;
import org.jxmapviewer.viewer.WaypointPainter;

public class MapPainterManualInput {
	
	public void paintMap(){
		final JXMapViewer mapViewer = new JXMapViewer();
		
		
				
				// Create a TileFactoryInfo for OpenStreetMap
				TileFactoryInfo info = new OSMTileFactoryInfo();
				DefaultTileFactory tileFactory = new DefaultTileFactory(info);
				tileFactory.setThreadPoolSize(8);
				mapViewer.setTileFactory(tileFactory);
				
				// Add interactions
				MouseInputListener mia = new PanMouseInputListener(mapViewer);
				mapViewer.addMouseListener(mia);
				mapViewer.addMouseMotionListener(mia);

				mapViewer.addMouseListener(new CenterMapListener(mapViewer));
				
				mapViewer.addMouseWheelListener(new ZoomMouseWheelListenerCursor(mapViewer));
				
				mapViewer.addKeyListener(new PanKeyListener(mapViewer));
				
				// Add a selection painter
				SelectionAdapter sa = new SelectionAdapter(mapViewer); 
				SelectionPainter sp = new SelectionPainter(sa); 
				mapViewer.addMouseListener(sa); 
				mapViewer.addMouseMotionListener(sa); 
				mapViewer.setOverlayPainter(sp);
				
				// Create tracks from the geo-positions of the voyages
				List<GeoPosition> all = new ArrayList<GeoPosition>();
				List<GeoPosition> track1 = new ArrayList<GeoPosition>();
				List<GeoPosition> track2 = new ArrayList<GeoPosition>();
				List<GeoPosition> track3 = new ArrayList<GeoPosition>();
				List<GeoPosition> track4 = new ArrayList<GeoPosition>();
				List<GeoPosition> track5 = new ArrayList<GeoPosition>();
				List<GeoPosition> track6 = new ArrayList<GeoPosition>();
				
				GeoPosition FBF = new GeoPosition(61.61, 5.08);
				GeoPosition VIS = new GeoPosition(61.37, 2.46);
				GeoPosition BID = new GeoPosition(61.28, 2.12);
				GeoPosition GFC = new GeoPosition(61.21, 2.27);
				GeoPosition GFB = new GeoPosition(61.20, 2.20);
				GeoPosition GFA = new GeoPosition(61.18, 2.19);
				GeoPosition VMO = new GeoPosition(61.04, 2.34);
				GeoPosition KVB = new GeoPosition(61.08, 2.50);
				GeoPosition VFB = new GeoPosition(60.78, 2.90);
				GeoPosition STB = new GeoPosition(61.21, 1.83);
				GeoPosition CPR = new GeoPosition(60.73, 3.65);
				GeoPosition WVE = new GeoPosition(60.75, 3.44);
				GeoPosition COI = new GeoPosition(60.81, 3.56);
				
				
				all.add(VIS);
				all.add(BID);
				all.add(GFC);
				all.add(GFB);
				all.add(GFA);
				all.add(VMO);
				all.add(KVB);
				all.add(VFB);
				all.add(STB);
				all.add(CPR);
				all.add(WVE);
				all.add(COI);
				
				// Create a compound painter that uses both the route-painter and the waypoint-painter
				List<Painter<JXMapViewer>> painters = new ArrayList<Painter<JXMapViewer>>();
				
				
				// Add all the routes to the painters
				track1.add(FBF);
				track1.add(VIS);
				track1.add(BID);
				track1.add(STB);
				track1.add(GFA);
				track1.add(GFB);
				track1.add(GFC);
				track1.add(FBF);
				RoutePainter routePainter1 = new RoutePainter(track1, Color.BLUE,0.3f);
				painters.add(routePainter1);
				
				track2.add(FBF);
				track2.add(COI);
				track2.add(CPR);
				track2.add(WVE);
				track2.add(VFB);
				track2.add(VMO);
				track2.add(KVB);
				track2.add(FBF);
				RoutePainter routePainter2 = new RoutePainter(track2, Color.RED,0.3f);
				painters.add(routePainter2);
				// Set the focus
//				mapViewer.zoomToBestFit(new HashSet<GeoPosition>(all), 0.7);
				mapViewer.setZoom(12);
				mapViewer.setAddressLocation(FBF);
				
				// Create waypoints from the geo-positions
				Set<MyWaypoint> waypoints = new HashSet<MyWaypoint>();
				waypoints.add(new MyWaypoint("FBF", Color.LIGHT_GRAY, FBF));
				waypoints.add(new MyWaypoint("VFB", Color.LIGHT_GRAY, VFB));
				waypoints.add(new MyWaypoint("KVB", Color.LIGHT_GRAY, KVB));
				waypoints.add(new MyWaypoint("VMO", Color.LIGHT_GRAY, VMO));
				waypoints.add(new MyWaypoint("GFC", Color.LIGHT_GRAY, GFC));
				waypoints.add(new MyWaypoint("GFB", Color.LIGHT_GRAY, GFB));
				waypoints.add(new MyWaypoint("GFA", Color.LIGHT_GRAY, GFA));
				waypoints.add(new MyWaypoint("BID", Color.LIGHT_GRAY, BID));
				waypoints.add(new MyWaypoint("VIS", Color.LIGHT_GRAY, VIS));
				waypoints.add(new MyWaypoint("STB", Color.LIGHT_GRAY, STB));
				waypoints.add(new MyWaypoint("CPR", Color.LIGHT_GRAY, CPR));
				waypoints.add(new MyWaypoint("VWE", Color.LIGHT_GRAY, WVE));
				waypoints.add(new MyWaypoint("COI", Color.LIGHT_GRAY, COI));

				// Create a waypoint painter that takes all the waypoints
				WaypointPainter<MyWaypoint> waypointPainter = new WaypointPainter<MyWaypoint>();
				waypointPainter.setWaypoints(waypoints);
				waypointPainter.setRenderer(new FancyWaypointRenderer());
				
				// Add the waypoints to the painters
				painters.add(waypointPainter);
				
				CompoundPainter<JXMapViewer> painter = new CompoundPainter<JXMapViewer>(painters);
				mapViewer.setOverlayPainter(painter);
				
				// Display the viewer in a JFrame
				final JFrame frame = new JFrame("Statoil solution");
				frame.add(new JLabel("Use left mouse button to pan and mouse wheel to zoom"), BorderLayout.NORTH);
				frame.getContentPane().add(mapViewer);
				frame.setSize(800, 600);
				frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				frame.setVisible(true);
				
				mapViewer.addPropertyChangeListener("zoom", new PropertyChangeListener()
				{
					@Override
					public void propertyChange(PropertyChangeEvent evt)
					{
						updateWindowTitle(frame, mapViewer);
					}
				});
				
				mapViewer.addPropertyChangeListener("center", new PropertyChangeListener()
				{
					@Override
					public void propertyChange(PropertyChangeEvent evt)
					{
						updateWindowTitle(frame, mapViewer);
					}
				});
				
				updateWindowTitle(frame, mapViewer);
	}
	
	
	protected static void updateWindowTitle(JFrame frame, JXMapViewer mapViewer)
	{
		double lat = mapViewer.getCenterPosition().getLatitude();
		double lon = mapViewer.getCenterPosition().getLongitude();
		int zoom = mapViewer.getZoom();
		
		frame.setTitle(String.format("Vessel schedule (%.2f / %.2f) - Zoom: %d", lat, lon, zoom)); 
	}
}
