package master;

import java.util.ArrayList;
/**
 * Contains the solutions object for a problem instance
 * @author Nils Albjerk
 * @author Teodor Danielsen
 * @author Stian Krey
 *
 */
public class Solution {
	
	ArrayList<Vessel> fleet;
	ArrayList<Node> removedNodes;
	int postponeCost;
	
	public int initNum;
	
	public Solution(ArrayList<Vessel> vessels, int postponecost){
		this.fleet = new ArrayList<Vessel>(vessels);
		this.removedNodes = new ArrayList<Node>();
		this.postponeCost = postponecost;
	}
	
	public Solution(ArrayList<Vessel> vessels, ArrayList<Node>nodes,int postponecost){
		this.fleet = vessels;
		this.removedNodes = nodes;
		this.postponeCost = postponecost;
	}
	
	
	public void setPostponeCost(int postponeCost){
		this.postponeCost = postponeCost;
	}
	
	
	/**
	 * Creates a copy of this solution object
	 */
	public Solution clone(){
		Solution solution = new Solution();
		solution.postponeCost = this.postponeCost;
		for (int i = 0; i < this.fleet.size(); i++) {
			solution.fleet.add(this.fleet.get(i));
		}
		for (int i = 0; i < removedNodes.size(); i++) {
			solution.removedNodes.add(removedNodes.get(i));
		}
		return solution;
	}
	
	public Solution (){
		this.fleet = new ArrayList<Vessel>();
		this.removedNodes = new ArrayList<Node>();
	}
	
	/**
	 * Creates a new solution object from an existing solution
	 * @param clone the existing solution
	 */
	public Solution(Solution clone){
		this.fleet = (ArrayList<Vessel>) clone.fleet.clone();
		this.removedNodes = (ArrayList<Node>) clone.removedNodes.clone();
		this.postponeCost = clone.postponeCost;
	}
	
	/**
	 * Calculates the total cost of this solution
	 * @return the total cost of this solution
	 */
	public double TotalCostInternal(){
		double totalCost = 0;
	
		for (int i = 0; i < fleet.size(); i++) {
			totalCost += fleet.get(i).cost();
		}
		for (int i = 0; i < removedNodes.size(); i++) {
			totalCost += removedNodes.get(i).getPriority()*postponeCost; // For bruk nr vi har prioritet p noder
		}
		return totalCost;
	}
	
	
	public void printResult(){
		for (int i = 0; i < fleet.size(); i++) {
			fleet.get(i).printVoyage();
			System.out.println("MinTime:			"+ fleet.get(i).minTime);
			System.out.println("Voyagae duration:		"+ (Math.round(fleet.get(i).voyageSailingTime()*100)/100d));
			System.out.println("MaxTime:			"+ fleet.get(i).maxTime);
			System.out.println("Ship delayed:			"+ (fleet.get(i).voyageSailingTime() + fleet.get(i).minTime > fleet.get(i).maxTime ));
			System.out.println("Ship delayed with:		"+ (Math.round(fleet.get(i).voyageSailingTime() + fleet.get(i).minTime  - fleet.get(i).maxTime )*100)/100d);
			System.out.println("Capacity ok:			"+ fleet.get(i).voyageCapOK());
			System.out.println("Spot:				"+ fleet.get(i).spot);
			double startLoad = 0;
			for (int j = 0; j < fleet.get(i).route.size(); j++) {
				if(fleet.get(i).route.get(j) instanceof DeliveryNode){
					DeliveryNode d = (DeliveryNode) fleet.get(i).route.get(j);
					startLoad = startLoad + d.getDemand(); 
				
				}	
			}
			System.out.println("Start load:			"+ startLoad);
			System.out.println("Capacity:			"+ fleet.get(i).getCapacity()); 
			System.out.println("Voyage cost:			" + Math.round(fleet.get(i).cost()*100)/100d);
			System.out.println("-------------------------------------");	
			}
		System.out.println("Nodes removed ");
		for (int i = 0; i < removedNodes.size(); i++) {
			System.out.print(removedNodes.get(i).getNodeNumber()+", ");
		}
		System.out.println("");for (int i = 0; i < removedNodes.size(); i++) {
			System.out.print(removedNodes.get(i).getNodeName()+", ");
		}
		System.out.println("");
		System.out.println("Total cost: "+ Math.round(TotalCostInternal()*100)/100d);
		}
	
	
	/**
	 * Checks if the constraints regarding capacity and time limits are adhered to, resulting in a feasible solution.
	 * @return a boolean that states if the solution is feasible
	 */
	public boolean allVesselsOK(){
		for (int i = 0; i < fleet.size(); i++) {
			if(!fleet.get(i).vesselOK()){
				return false;
			}
		}
		return true;
		
		
	}
	
	public int nNodesSolution(){
		int n = 0;
		for (int i = 0; i < fleet.size(); i++) {
			for (int j = 1; j < fleet.get(i).route.size()-1; j++) {
				n++;
			}
		}
		return n;
	}
	
	public void routeLoadPrint(){
		for (int f = 0; f < fleet.size(); f++) {
			if (fleet.get(f).route.size()>2) {
				System.out.println("Vessel number: "+ f);
				double startLoad = 0;

				for (int i = 0; i < fleet.get(f).route.size(); i++) {
					if(fleet.get(f).route.get(i) instanceof DeliveryNode){
						DeliveryNode d = (DeliveryNode) fleet.get(f).route.get(i);
						startLoad = startLoad + d.getDemand(); 
					}	
				}
				System.out.println("Load from "+fleet.get(f).route.get(0).getNodeName()+" to "+ fleet.get(f).route.get(1).getNodeName()+ " is: "+ startLoad);
				double load = startLoad;
				//sjekk for alle steg i ruta.
				for (int i = 1; i < fleet.get(f).route.size()-1; i++) {	// starts with node 1 in the route,
					if(fleet.get(f).route.get(i) instanceof DeliveryNode){
						DeliveryNode d = (DeliveryNode) fleet.get(f).route.get(i);
						load = load - d.getDemand();
					}
					if(fleet.get(f).route.get(i) instanceof PickupNode){
						PickupNode p = (PickupNode) fleet.get(f).route.get(i);
						load = load + p.getBackload();
					}
					System.out.println("Load from "+ fleet.get(f).route.get(i).getNodeName()+" to "+ fleet.get(f).route.get(i+1).getNodeName()+ " is: "+load);
				}
				System.out.println("--------------");
			}
			else{
				System.out.println("Vessel "+f+" is not being used");
			}
			

		}
		
	}
	
		
	}
	

