package master;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Random;

/**
 * The variable neighborhood search heuristic, with rearrangement of solutions
 * @author Nils Albjerk
 * @author Teodor Danielsen
 * @author Stian Krey
 *
 */
public class VNS {
		private Moves moves = new Moves();
		private Random rnSwap = new Random();
		private Random rnTwoOpt = new Random();
		private Random rnThreeOpt = new Random();
		private Random rnInsert = new Random();
		private Random rnShift = new Random();
		private Random rnCross = new Random();
		private Random rnReplace = new Random();
		private Random rnExchange = new Random();
		private Random rnSibling = new Random();
		private Random rnEjection = new Random();
		private ArrayList<Node> moveTabuList = new ArrayList<Node>();
		private ArrayList<Node> dropTabuList = new ArrayList<Node>();
		private ArrayList<Node> dropAndReinsertTabuList = new ArrayList<Node>();
		private ArrayList<Node> reinsertTabuList = new ArrayList<Node>();
		private ArrayList<Node> siblingTabuList = new ArrayList<Node>();
		private int k;
		private int sizeOfRemovedNodes;
		boolean reset = true;
		boolean dropAndReinsertByLoad = true;
		boolean stopSiblingReunion = false;
		boolean stopReinsert = false;
		
		/**
		 * The heuristic used to improve the initial solution provided
		 * @param solution the solution to be improved
		 * @param nValue the value used to calculate the nMax parameter
		 * @param pValue the value used to calculate the pMax parameter
		 * @param sValue the value used to calculate the sMax parameter
		 * @param fname the problem instance file
		 * @param outputFileLocation the location where the solution should be stored
		 * @return the best found solution with regard to total cost of sailing the routes, delays and postponing orders
		 * @throws IOException
		 */
		public Solution VNSAlgortihm(Solution solution, double nValue, double pValue, int sValue, String fname, String outputFileLocation) throws IOException{
			long startTime = System.nanoTime();
			String impFile = outputFileLocation+"/"+fname+"_improvements.txt";
			/*
			 * Printing to file for improvements
			 */
			File file_imp = new File(impFile);
			if (!file_imp.exists()) {
				file_imp.createNewFile();
			}
			FileWriter fw_all = new FileWriter(file_imp.getAbsoluteFile(), true);
			BufferedWriter bw_all = new BufferedWriter(fw_all);
			long currTime3 = System.nanoTime();
			bw_all.write("Initial		" + Math.round(solution.TotalCostInternal()*100)/100d + "	after	"+ (currTime3-startTime)/1000000 + "	ms");
			bw_all.newLine();
			
			
			ArrayList<Vessel> vessels = solution.fleet;
			int nNodes = 2;
			for (int i = 0; i < vessels.size(); i++) {
				for (int j = 1; j < vessels.get(i).route.size()-1; j++) {
					nNodes ++;		
				}
			}
			for (int i = 0; i < solution.removedNodes.size(); i++) {
				nNodes++;
			}
			
			ArrayList<Node> initRemovedNodes = new ArrayList<Node>();
			for (int i = 0; i < solution.removedNodes.size(); i++) {
				initRemovedNodes.add(solution.removedNodes.get(i));
			}
			
			
			int pMax = ((int) Math.ceil(pValue*nNodes));
			int sMax = sValue* nNodes;
			int kMax = 8;
			int mMax = 8;
			if(isDelayed(solution.fleet)){
				kMax = 9;
				mMax = 9;
			}
			int nMax = ((int) Math.ceil(((nNodes-2)/2)*nValue));

			Solution bestSolution = solutionCopy(solution);	
			Solution tempSolutionAfterDrop = solutionCopy(bestSolution);
			
			int loadCounter = 0;
			// The outer loop, controlling how many times the solutions is rearranged
			for(int n = 0; n<nMax; n++){
				/*
				 * Printing the content of all tabu lists used
				 */
				System.out.println("");
				System.out.println("**********************************");
				System.out.println("Starting Rearrangement phase");
				System.out.println("");
				System.out.println("dropAndReinsertTabuList contains:");
				for (int i = 0; i < dropAndReinsertTabuList.size(); i++) {
					System.out.print(dropAndReinsertTabuList.get(i).getNodeNumber()+" "+dropAndReinsertTabuList.get(i).getNodeName()+",");
				}
				System.out.println("");
				System.out.println("dropTabuList contains:");
				for (int i = 0; i < dropTabuList.size(); i++) {
					System.out.print(dropTabuList.get(i).getNodeNumber()+" "+dropTabuList.get(i).getNodeName()+",");
				}
				System.out.println("");
				System.out.println("reinsertTabuList contains:");
				for (int i = 0; i < reinsertTabuList.size(); i++) {
					System.out.print(reinsertTabuList.get(i).getNodeNumber()+" "+reinsertTabuList.get(i).getNodeName()+",");
				}
				System.out.println("");
				System.out.println("siblingTabuList contains:");
				for (int i = 0; i < siblingTabuList.size(); i++) {
					System.out.print(siblingTabuList.get(i).getNodeNumber()+" "+siblingTabuList.get(i).getNodeName()+",");
				}
				System.out.println("");
				System.out.println("moveTabuList contains:");
				for (int i = 0; i < moveTabuList.size(); i++) {
					System.out.print(moveTabuList.get(i).getNodeNumber()+" "+moveTabuList.get(i).getNodeName()+",");
				}
				System.out.println("");
				System.out.println("");
				if(n == 0){
					
				}else if(isDelayed(bestSolution.fleet)&& n > 0){
					// Most likely case with speed reduction
					dropAndReinsertByLoad = false;
					if(!stopSiblingReunion){
						System.out.println("Running sibling reunion");
						tempSolutionAfterDrop = siblingReunion(bestSolution);
						if(stopSiblingReunion){
							System.out.println("Stopping sibling reunion, reset n for sibling move");
							n = (int) Math.ceil(nMax/2);
							reset = false;
							System.out.println("Running sibling move");
							tempSolutionAfterDrop = siblingPairMover(bestSolution);
						}
					}else{
						System.out.println("Running sibling move");
						tempSolutionAfterDrop = siblingPairMover(bestSolution);
					}
					if(n == nMax-1 && reset){
						System.out.println("nMax reached, reset for sibling move");
						n = (int) Math.ceil(nMax/2);
						stopSiblingReunion = true;
						reset = false;
					}
					
				}else if(!isDelayed(bestSolution.fleet) && n>0 && !bestSolution.removedNodes.isEmpty()){
					System.out.println("Load counter = "+loadCounter);
					// Most likely case with triple loads
					if(loadCounter<Math.ceil(0.4*nMax) && dropAndReinsertByLoad){
						System.out.println("Run drop and reinsert");
						tempSolutionAfterDrop = dropAndReinsertByLoad(bestSolution);
						if(!dropAndReinsertByLoad){
							System.out.println("Run reinsert");
							tempSolutionAfterDrop = reinsertByLoadSmart(bestSolution);
						}
					}else if(!dropAndReinsertByLoad && loadCounter < sizeOfRemovedNodes){
						if(n == nMax-1){
							n -= 1;
						}
						System.out.println("Run reinsert");
						tempSolutionAfterDrop = reinsertByLoadSmart(bestSolution);
						if(stopReinsert){
							// Print results to improvements file
							long currTime = System.nanoTime();
							bw_all.write("Final		" + Math.round(bestSolution.TotalCostInternal()*100)/100d + "	after	"+ (currTime-startTime)/1000000 + "	ms");
							bw_all.newLine();
							bw_all.newLine();
							bw_all.close();
							return bestSolution;
						}
					}
					loadCounter++;
				}else if(n>2&&!isDelayed(bestSolution.fleet) && bestSolution.removedNodes.isEmpty()){
					// Print results to improvement file
					long currTime = System.nanoTime();
					bw_all.write("Final		" + Math.round(bestSolution.TotalCostInternal()*100)/100d + "	after	"+ (currTime-startTime)/1000000 + "	ms");
					bw_all.newLine();
					bw_all.newLine();
					bw_all.close();
					return bestSolution;
				}

				ArrayList<Vessel>pBest = vesselsCloning(tempSolutionAfterDrop.fleet); 
				ArrayList<Vessel>tempPBest = vesselsCloning(tempSolutionAfterDrop.fleet); 
				ArrayList<Vessel>vesselsCopy = vesselsCloning(tempSolutionAfterDrop.fleet);

					System.out.println("Starting Improvement phase");
					//The loop controlling the number of perturbations
					for (int p = 0; p < pMax; p++) {
						//The loop controlling the number of shakings
						for (int s = 0; s < sMax; s++) {
							vesselsCopy = vesselsCloning(runK(kMax, vesselsCopy, mMax));
							vesselsCopy= vesselsCloning(vesselSwitch(vesselsCopy));
							if(TotalCost(vesselsCopy)<TotalCost(pBest) && allVesselsOK(vesselsCopy)){
								vesselsCopy= siblingOrder(vesselsCopy);
								pBest = vesselsCloning(vesselsCopy);
								tempPBest= vesselsCloning(vesselsCopy);
								p = 0;
								s = 0;
							}
						}
						vesselsCopy= vesselsCloning(ejectionChain(tempPBest));
	
	
					}
					System.out.println("Ending Improvement phase");
					tempSolutionAfterDrop.fleet = vesselsCloning(pBest);
					System.out.println("Temp solution cost is "+Math.round(tempSolutionAfterDrop.TotalCostInternal()*100)/100d+", feasible solution: "+allVesselsOK(tempSolutionAfterDrop.fleet));
					
					if(tempSolutionAfterDrop.TotalCostInternal() < bestSolution.TotalCostInternal() && allVesselsOK(tempSolutionAfterDrop.fleet)){
						if(dropAndReinsertByLoad && n>0){
							System.out.println("Successfully drop and reinsert, set n = 1");
							n=1;
							loadCounter = 0;
						}
						
						bestSolution = solutionCopy(tempSolutionAfterDrop);

						
						long currTime = System.nanoTime();
						// Print results to improvements file
						bw_all.write("Improved	" + Math.round(bestSolution.TotalCostInternal()*100)/100d + "	after	"+ (currTime-startTime)/1000000 + "	ms");
						bw_all.newLine();
						System.out.println("Improved solution found with value						" + Math.round(bestSolution.TotalCostInternal()*100)/100d + " after "+ (currTime-startTime)/1000000 + " ms");
				
					}else if(dropAndReinsertByLoad && loadCounter>=Math.ceil(0.4*nMax)){
						System.out.println("Finished drop and reinsert, resetting load counter for reinserting");
						dropAndReinsertByLoad = false;
						sizeOfRemovedNodes = bestSolution.removedNodes.size();
						loadCounter = 0;
					}
	
					System.out.println("Best solution is "+Math.round(bestSolution.TotalCostInternal()*100)/100d);
					System.out.println("Ending Rearrangement phase");
			}
			// Print results to improvements file
			long currTime = System.nanoTime();
			bw_all.write("Final		" + Math.round(bestSolution.TotalCostInternal()*100)/100d + "	after	"+ (currTime-startTime)/1000000 + "	ms");
			bw_all.newLine();
			bw_all.newLine();
			bw_all.close();
			return bestSolution;

		}
		
		/**
		 * The shaking move used to provide the local search with new starting points
		 * @param kMax the number of neighborhood moves to use in the shaking
		 * @param vessels the list of vessels in the solution
		 * @param mMax the number of neighborhood moves to use in the local search if the shaking solution is non-improving
		 * @return a list of vessels with the best found routes in the shaking
		 */
		public ArrayList<Vessel> runK(int kMax, ArrayList<Vessel>vessels, int mMax){
			ArrayList<Vessel> vesselsCopy = vessels;
			ArrayList<Vessel> kBest = vesselsCloning(vessels);
			for (k = 0; k < kMax; k++) {
				vesselsCopy = vesselsCloning(kBest);
				switch (k) {
				case 5:
					vesselsCopy = intra_ThreeOpt(vesselsCopy);
					kBest = checkK(kBest, vesselsCopy, mMax);
					break;
				case 1:
					vesselsCopy = intra_Swap(vesselsCopy);
					kBest = checkK(kBest, vesselsCopy, mMax);
					break;
				case 2:
					vesselsCopy = inter_Shift(vesselsCopy);
					kBest = checkK(kBest, vesselsCopy, mMax);
					break;
				case 4:
					vesselsCopy = intra_TwoOpt(vesselsCopy);
					kBest = checkK(kBest, vesselsCopy, mMax);
					break;
				case 7:
					vesselsCopy = inter_Exchange(vesselsCopy);
					kBest = checkK(kBest, vesselsCopy, mMax);
					break;
				case 6:
					vesselsCopy = inter_Cross(vesselsCopy);
					kBest = checkK(kBest, vesselsCopy, mMax);
					break;
				case 0:
					vesselsCopy = intra_Insert(vesselsCopy);
					kBest = checkK(kBest, vesselsCopy, mMax);
					break;
				case 3:
					vesselsCopy = inter_Replace(vesselsCopy);
					kBest = checkK(kBest, vesselsCopy, mMax);
					break;
				case 8:
					vesselsCopy = inter_SiblingPairMover(vesselsCopy);
					kBest = checkK(kBest, vesselsCopy, mMax);
					break;
				default:
					break;
				}
				
			}
			return kBest;
		}
		
		/**
		 * The local search used to improve solutions found in the shaking step
		 * @param mMax the number of neighborhood moves to use in the local search
		 * @param vessels the list of vessels in the solution
		 * @param best the best found solution in the shaking step that initialized the local search
		 * @return a list of vessels with the best found routes in the local search
		 */
		public ArrayList<Vessel> runM(int mMax, ArrayList<Vessel>vessels, ArrayList<Vessel> best){
			ArrayList<Vessel> mBest = vesselsCloning(best);
			ArrayList<Vessel> vesselsCopy = vessels;
			for (int m = 0; m < mMax; m++) {
				vesselsCopy = vesselsCloning(mBest);
				switch (m) {
				case 5:
					vesselsCopy = intra_ThreeOpt(vesselsCopy);
					mBest = checkM(mBest, vesselsCopy);
					break;
				case 1:
					vesselsCopy = intra_Swap(vesselsCopy);
					mBest = checkM(mBest, vesselsCopy);
					break;
				case 2:
					vesselsCopy = inter_Shift(vesselsCopy);
					mBest = checkM(mBest, vesselsCopy);
					break;
				case 4:
					vesselsCopy = intra_TwoOpt(vesselsCopy);
					mBest = checkM(mBest, vesselsCopy);
					break;
				case 7:
					vesselsCopy = inter_Exchange(vesselsCopy);
					mBest = checkM(mBest, vesselsCopy);
					break;
				case 6:
					vesselsCopy = inter_Cross(vesselsCopy);
					mBest = checkM(mBest, vesselsCopy);
					break;
				case 0:
					vesselsCopy = intra_Insert(vesselsCopy);
					mBest = checkM(mBest, vesselsCopy);
					break;
				case 3:
					vesselsCopy = inter_Replace(vesselsCopy);
					mBest = checkM(mBest, vesselsCopy);
					break;
				case 8:
					vesselsCopy = inter_SiblingPairMover(vesselsCopy);
					mBest = checkM(mBest, vesselsCopy);
					break;
				default:
					break;
				}
			}
			
			return mBest;
			
		}
		
		/**
		 * Checks the routes found after a local search move against the best known solution in the local search
		 * @param best the best known solution found in the local search so far
		 * @param current the solution obtained after performing the current neighborhood move
		 * @return the best one of the two solutions compared
		 */
		public ArrayList<Vessel> checkM(ArrayList<Vessel>best,ArrayList<Vessel>current){
			if(TotalCost(current)<TotalCost(best) && allVesselsOK(current)){
				ArrayList<Vessel>newBest = vesselsCloning(current);
				return newBest;
			}
			return best;
		}
		
		/**
		 * Checks the routes found after a shaking move against the best known solution in the shaking step. 
		 * If it is non-improving, a local search is run on the solution, if not, the next shaking move is performed
		 * @param best the best known solution found in the shaking so far
		 * @param current the solution obtained after performing the current neighborhood move
		 * @param mMax the number of neighborhood moves to use in the local search
		 * @return the best one of the two solutions compared
		 */
		public ArrayList<Vessel> checkK(ArrayList<Vessel> best, ArrayList<Vessel> current, int mMax){
			
			if(TotalCost(current)<TotalCost(best) && allVesselsOK(current)){
				ArrayList<Vessel>newBest = vesselsCloning(current);
				k = 0;
				return newBest;
				
			}else{
				ArrayList<Vessel> temp = vesselsCloning(current);
				ArrayList<Vessel> returned = vesselsCloning(runM(mMax, temp, best));
				if(TotalCost(returned)<TotalCost(best) && allVesselsOK(returned)){
					ArrayList<Vessel>newBest = vesselsCloning(returned);
					k = 0;
					return newBest;
				}
			}
			return best;
		}
		
		
		/**
		 * Performs the swap move on a random vessel 
		 * @param the list of vessels in the solution
		 * @return the list of vessels with the routes obtained by the move
		 */
		public ArrayList<Vessel> intra_Swap(ArrayList<Vessel> vessels){
					
			ArrayList<Vessel> vesselsCopy2 = vesselsCloning(vessels);
			int rand = rnSwap.nextInt(vesselsCopy2.size());

			vesselsCopy2.get(rand).route = moves.swap(vesselsCopy2.get(rand));

			return vesselsCopy2;
		}
		
		/**
		 * Performs the two-opt move on a random vessel 
		 * @param the list of vessels in the solution
		 * @return the list of vessels with the routes obtained by the move
		 */
		public ArrayList<Vessel> intra_TwoOpt(ArrayList<Vessel> vessels){
					
			ArrayList<Vessel> vesselsCopy2 = vesselsCloning(vessels);
			int rand = rnTwoOpt.nextInt(vesselsCopy2.size());

			vesselsCopy2.get(rand).route = moves.twoOpt(vesselsCopy2.get(rand));

			return vesselsCopy2;
		}
		
		/**
		 * Performs the three-opt move on a random vessel 
		 * @param the list of vessels in the solution
		 * @return the list of vessels with the routes obtained by the move
		 */
		public ArrayList<Vessel> intra_ThreeOpt(ArrayList<Vessel> vessels){
					
			ArrayList<Vessel> vesselsCopy2 = vesselsCloning(vessels);
			int rand = rnThreeOpt.nextInt(vesselsCopy2.size());

			vesselsCopy2.get(rand).route = moves.threeOpt(vesselsCopy2.get(rand));

			return vesselsCopy2;
		}
		
		/**
		 * Performs the insert move on a random vessel 
		 * @param the list of vessels in the solution
		 * @return the list of vessels with the routes obtained by the move
		 */
		public ArrayList<Vessel> intra_Insert(ArrayList<Vessel> vessels){
					
			ArrayList<Vessel> vesselsCopy2 = vesselsCloning(vessels);
			int rand = rnInsert.nextInt(vesselsCopy2.size());

			vesselsCopy2.get(rand).route = moves.insert(vesselsCopy2.get(rand));

			return vesselsCopy2;
		}
		
		
			
//---------------------------------------------------------------------------------------------

		
		
		/**
		 * Performs the shift move on two random vessels
		 * @param the list of vessels in the solution
		 * @return the list of vessels with the routes obtained by the move
		 */
		public ArrayList<Vessel> inter_Shift(ArrayList<Vessel> vessels){

			ArrayList<Vessel> vesselsCopy2 = vesselsCloning(vessels);
			int rand1 = rnShift.nextInt(vesselsCopy2.size());
			int rand2 = rnShift.nextInt(vesselsCopy2.size());
			while(rand1 == rand2){
				rand2 = rnShift.nextInt(vesselsCopy2.size());
			}

			ArrayList<ArrayList<Node>> new_routes = moves.shift(vesselsCopy2.get(rand1), vesselsCopy2.get(rand2));
			vesselsCopy2.get(rand1).route = new_routes.get(0);
			vesselsCopy2.get(rand2).route = new_routes.get(1);
		
			return vesselsCopy2;
		}
		
		/**
		 * Performs the cross move on two random vessels
		 * @param the list of vessels in the solution
		 * @return the list of vessels with the routes obtained by the move
		 */
		public ArrayList<Vessel> inter_Cross(ArrayList<Vessel> vessels){

			ArrayList<Vessel> vesselsCopy2 = vesselsCloning(vessels);

			int rand1 = rnCross.nextInt(vesselsCopy2.size());
			int rand2 = rnCross.nextInt(vesselsCopy2.size());
			while(rand1 == rand2){
				rand2 = rnCross.nextInt(vesselsCopy2.size());
			}

			ArrayList<ArrayList<Node>> new_routes = moves.cross(vesselsCopy2.get(rand1), vesselsCopy2.get(rand2));
			vesselsCopy2.get(rand1).route = new_routes.get(0);
			vesselsCopy2.get(rand2).route = new_routes.get(1);

			return vesselsCopy2;
		}
		
		/**
		 * Performs the replace move on two random vessels
		 * @param the list of vessels in the solution
		 * @return the list of vessels with the routes obtained by the move
		 */
		public ArrayList<Vessel> inter_Replace(ArrayList<Vessel> vessels){
			
			ArrayList<Vessel> vesselsCopy2 = vesselsCloning(vessels);

			int rand1 = rnReplace.nextInt(vesselsCopy2.size());
			int rand2 = rnReplace.nextInt(vesselsCopy2.size());
			while(rand1 == rand2){
				rand2 = rnReplace.nextInt(vesselsCopy2.size());
			}

			ArrayList<ArrayList<Node>> new_routes = moves.replace(vesselsCopy2.get(rand1), vesselsCopy2.get(rand2));
			vesselsCopy2.get(rand1).route = new_routes.get(0);
			vesselsCopy2.get(rand2).route = new_routes.get(1);
		
			return vesselsCopy2;
		}
		
		/**
		 * Performs the exchange move on two random vessels
		 * @param the list of vessels in the solution
		 * @return the list of vessels with the routes obtained by the move
		 */
		public ArrayList<Vessel> inter_Exchange(ArrayList<Vessel> vessels){

			ArrayList<Vessel> vesselsCopy2 = vesselsCloning(vessels);

			int rand1 = rnExchange.nextInt(vesselsCopy2.size());
			int rand2 = rnExchange.nextInt(vesselsCopy2.size());
			while(rand1 == rand2){
				rand2 = rnExchange.nextInt(vesselsCopy2.size());
			}

			ArrayList<ArrayList<Node>> new_routes = moves.shift(vesselsCopy2.get(rand1), vesselsCopy2.get(rand2));
			vesselsCopy2.get(rand1).route = new_routes.get(0);
			vesselsCopy2.get(rand2).route = new_routes.get(1);

			return vesselsCopy2;
		}
		
		/**
		 * Performs the sibling move on the sibling that is furthest away from all other nodes in it's route
		 * @param the list of vessels in the solution
		 * @return the list of vessels with the routes obtained by the move
		 */
		public ArrayList<Vessel> inter_SiblingPairMover(ArrayList<Vessel> vessels){
			ArrayList<Vessel> vesselsCopy2 = vesselsCloning(vessels);
			int vessel = rnSibling.nextInt(vesselsCopy2.size());
			double longestSiblingSailingTime = 0;
			Node furthest = null;
			
			for (int j = 1; j < vesselsCopy2.get(vessel).route.size()-1; j++) {
				double sailingTime = distanceToAllInRoute(vesselsCopy2.get(vessel).route, vesselsCopy2.get(vessel).route.get(j));
				if(sailingTime > longestSiblingSailingTime){
					longestSiblingSailingTime = sailingTime;
					furthest = vesselsCopy2.get(vessel).route.get(j);
				}
			}

			if(furthest != null){
				vesselsCopy2.get(vessel).route.remove(furthest);

				for (int i = 0; i < vesselsCopy2.size(); i++) {
					if(vesselsCopy2.get(i).route.contains(furthest.getSibling())){
						vesselsCopy2.get(i).route.remove(furthest.getSibling());
					}
				}
				double shortestSailingTime = 10000;
				int reinsertVessel = 0;
				for (int i = 0; i < vesselsCopy2.size(); i++) {
					if(i == vessel){
						continue;
					}
					double sailingTime = distanceToAllInRoute(vesselsCopy2.get(i).route, furthest);
					if(sailingTime<shortestSailingTime){
						shortestSailingTime = sailingTime;
						reinsertVessel= i;
					}
					
				}
				vesselsCopy2.get(reinsertVessel).route.add(1, furthest);
				if(furthest.getSibling() != null){
					vesselsCopy2.get(reinsertVessel).route.add(1, furthest.getSibling());
				}
			}
			
			return vesselsCopy2;
			
		}
		
//----------------------------------------------------------------------------------------------------------------------------------------		
		
		/**
		 * Calculates the cost of the vessels being used
		 * @param vessels the list of vessels in the solution
		 * @return the cost of the vessels in use
		 */
		public double TotalCost(ArrayList<Vessel> vessels){
			double totalCost = 0;
		
			for (int i = 0; i < vessels.size(); i++) {
				totalCost += vessels.get(i).cost();
			}
			return totalCost;
		}
		
		/**
		 * Checks if all vessels adheres to the time limit and capacity constraints
		 * @param vessels the list of vessels in the solution
		 * @return a boolean stating if all the vessels are feasible 
		 */
		public boolean allVesselsOK(ArrayList<Vessel> vessels){
			for (int i = 0; i < vessels.size(); i++) {
				if(!vessels.get(i).vesselOK()){
					return false;
				}
			}
			return true;
			
			
		}
		/**
		 * Creates a clone of the lists of vessels
		 * @param vessels the list of vessels in the solution
		 * @return the clone of the list of vessels
		 */
		public ArrayList<Vessel> vesselsCloning(ArrayList<Vessel> vessels){
			ArrayList<Vessel> vesselsClone = new ArrayList<Vessel>();
			
			for (int i = 0; i < vessels.size(); i++) {
				Vessel tempVes = new Vessel(vessels.get(i));
				vesselsClone.add(tempVes);
			}
			
			return vesselsClone;
		}
		
		public void printResult(ArrayList<Vessel> vessels){
			System.out.println("Final reults: ");
			for (int i = 0; i < vessels.size(); i++) {
				vessels.get(i).printVoyage();
				System.out.println("MinTime:			"+ vessels.get(i).minTime);
				System.out.println("Voyagae duration:		"+ Math.round(vessels.get(i).voyageSailingTime()));
				System.out.println("MaxTime:			"+ vessels.get(i).maxTime);
				System.out.println("Capacity ok?:	"+ vessels.get(i).voyageCapOK());
				System.out.println("Spot:		"+ vessels.get(i).spot);
				System.out.println("Voyage cost:		" + vessels.get(i).cost());
				System.out.println("-----------------------------");	
				}
			System.out.println("Total cost: "+ TotalCost(vessels));
			}
		
		/**
		 * Moves a node, and it's sibling (if it is located on the same vessel) to the next vessel in the list. 
		 * @param vessels the list of vessels in the solution
		 * @return the list of vessels with the routes obtained by the ejection chain
		 */
		public ArrayList<Vessel> ejectionChain(ArrayList<Vessel> vessels){
			for (int i = 0; i < vessels.size(); i++) {
				if(vessels.get(i).route.size()<=2){
					continue;
				}

				int max = vessels.get(i).route.size()-2;
				int min = 1;
				int removePos = rnEjection.nextInt(max-min+1)+min;
				boolean siblingRemoved = false;
				Node node = vessels.get(i).route.get(removePos);
				Node sibling = node.getSibling();
				if(node.getSibling() != null && vessels.get(i).route.contains(node.getSibling())){
					vessels.get(i).route.remove(sibling);
					siblingRemoved = true;
				}
				vessels.get(i).route.remove(node);
				int addPos = 1;
				if(i == vessels.size()-1){
					if(vessels.get(0).route.size()>2){

						int insertMax = vessels.get(0).route.size()-2;
						int insertMin = 1;
						addPos = rnEjection.nextInt(insertMax-insertMin+1)+insertMin;
					}
					vessels.get(0).route.add(addPos, node);;
					if(sibling != null && siblingRemoved){
						vessels.get(0).route.add(addPos,sibling);
					}
				}else{
					if(vessels.get(i+1).route.size()>2){

						int insertMax = vessels.get(i+1).route.size()-2;
						int insertMin = 1;
						addPos = rnEjection.nextInt(insertMax-insertMin+1)+insertMin;
					}
					vessels.get(i+1).route.add(addPos, node);
					if(sibling != null && siblingRemoved){
						vessels.get(i+1).route.add(addPos, sibling);
					}
				}
			}
			return vessels;
		}
		
		/**
		 * Creates a copy of the given solution
		 * @param solution the solution to be copied
		 * @return a copy of the solution
		 */
		private Solution solutionCopy(Solution solution){
			ArrayList<Vessel> vessels = vesselsCloning(solution.fleet);
			ArrayList<Node> nodes = new ArrayList<Node>();
			for (int i = 0; i < solution.removedNodes.size(); i++) {
				nodes.add(solution.removedNodes.get(i));
			}
			Solution sol = new Solution(vessels,nodes,solution.postponeCost);
			return sol;
		}
		
		/**
		 * Checks if any vessel is delayed more than two hours
		 * @param vessels the list of vessels in the solution
		 * @return a boolean stating if any vessel is delayed more than two hours
		 */
		public boolean isDelayed(ArrayList<Vessel> vessels){
			boolean delay = false;
			for (int i = 0; i < vessels.size()-1; i++) {
				double vesselSailTime = vessels.get(i).voyageSailingTime() + vessels.get(i).minTime;
				if( vesselSailTime > vessels.get(i).maxTime+2){
					delay = true;
					return delay;
				}
			}
			return delay;
		}
		
		/**
		 * Drops the node with the largest load on all of the vessels, and exchanges it with the node in the removedNodes 
		 * list with the smallest load
		 * @param solution the solution to be rearranged
		 * @return the solution with rearrangements if any were done
		 */
		public Solution dropAndReinsertByLoad(Solution solution){
			if(solution.removedNodes.isEmpty()){
				return solution;
			}
			Solution sol = solutionCopy(solution);
			double lowestDroppedLoad = 1000;
			Node reinsert = null;
			for (int i = 0; i < sol.removedNodes.size(); i++){
				double load = 0;
				if(dropAndReinsertTabuList.contains(sol.removedNodes.get(i))){
					continue;
				}
				if(sol.removedNodes.get(i) instanceof DeliveryNode){
					DeliveryNode d = (DeliveryNode) sol.removedNodes.get(i);
					load = d.getDemand();
				}
				if(sol.removedNodes.get(i) instanceof PickupNode){
					PickupNode p = (PickupNode) sol.removedNodes.get(i);
					load = p.getBackload();
				}
				if(load<lowestDroppedLoad){
					lowestDroppedLoad = load;
					reinsert = sol.removedNodes.get(i);
				}
			}
			if(reinsert == null){
				System.out.println("No node to reinsert");
				dropAndReinsertByLoad = false;
				return sol;
			}
			ArrayList<Node> allNodes = allNodesInRoutes(sol);

			double highestOnboardLoad = 0;
			int pos1 = 0;
			for (int i = 0; i < allNodes.size(); i++){
				if(dropTabuList.contains(allNodes.get(i))){
					continue;
				}
				double load = 0;
				if(allNodes.get(i) instanceof DeliveryNode){
					DeliveryNode d = (DeliveryNode) allNodes.get(i);
					load = d.getDemand();
				}
				if(allNodes.get(i) instanceof PickupNode){
					PickupNode p = (PickupNode) allNodes.get(i);
					load = p.getBackload();
				}
				if(load>highestOnboardLoad){
					highestOnboardLoad = load;
					pos1 = i;
				}
			}

			Node remove = allNodes.get(pos1);
			for (int i = 0; i < sol.fleet.size(); i++) {
				if(sol.fleet.get(i).route.contains(remove)){
					System.out.println("Removing node "+remove.getNodeNumber()+" "+remove.getNodeName()+" from vessel "+i);
					System.out.println("Reinserting node "+reinsert.getNodeNumber()+" "+reinsert.getNodeName()+" into vessel "+i);
					sol.fleet.get(i).route.remove(remove);
					sol.removedNodes.add(remove);
					if(reinsert instanceof DeliveryNode){
						sol.fleet.get(i).route.add(1, reinsert);
						sol.removedNodes.remove(reinsert);
					}
					if(reinsert instanceof PickupNode){
						int size = sol.fleet.get(i).route.size();
						if(size == 2){
							sol.fleet.get(i).route.add(1, reinsert);
							sol.removedNodes.remove(reinsert);
						}else{
							sol.fleet.get(i).route.add(size-2, reinsert);
							sol.removedNodes.remove(reinsert);
						}
						
					}
					dropTabuList.add(remove);
					dropAndReinsertTabuList.add(reinsert);
				}
			}
			return sol;
		}
		
		/**
		 * Finds the node amongst the removedNodes that gives the lowest total cost for the solution if it is reinserted
		 * @param solution the solution to be rearranged
		 * @return the solution with rearrangements if any were done
		 */
		public Solution reinsertByLoadSmart(Solution solution){
			if(solution.removedNodes.isEmpty()){
				System.out.println("Nothing to reinsert");
				return solution;
			}
			Solution sol = solutionCopy(solution);
			Node reinsert = null;
			Node bestReinsert = null;
			int reinsertVessel = 0;
			int reinsertPos = 0;
			double bestCost = 10000000;
			for (int k = 0; k < sol.removedNodes.size(); k++){
				if(reinsertTabuList.contains(sol.removedNodes.get(k))){
					continue;
				}
				reinsert = sol.removedNodes.get(k);
				for (int i = 0; i < sol.fleet.size(); i++) {
					for (int j = 1; j < sol.fleet.get(i).route.size()-1; j++) {
						double cost = sol.fleet.get(i).voyageCostAdd(reinsert, j);
						if(cost < bestCost && sol.fleet.get(i).voyageCapOKAdd(j, reinsert)){
							bestCost = cost;
							reinsertVessel = i;
							reinsertPos = j;
							bestReinsert = reinsert;
						}
					}

				}
			}
			
			
			if(reinsertPos==0){
				for (int i = 0; i < sol.removedNodes.size(); i++) {
					if(!reinsertTabuList.contains(sol.removedNodes.get(i))){
						bestReinsert = sol.removedNodes.get(i);
						break;
					}
				}
				if(bestReinsert == null){
					System.out.println("Nothing to reinsert");
					stopReinsert = true;
					return solution; 
				}
				reinsertVessel = sol.fleet.size()-1;
				if(bestReinsert instanceof PickupNode){
					reinsertPos = sol.fleet.get(reinsertVessel).route.size()-1;
				}
				if(bestReinsert instanceof DeliveryNode){
					reinsertPos = 1;
				}
				System.out.println("No feasible position found for reinserting");
				stopReinsert = true;
				return solution;
			}

			System.out.println("Reinserting "+bestReinsert.getNodeNumber()+ " "+bestReinsert.getNodeName()+" into "+reinsertVessel+" at pos "+reinsertPos);
			sol.fleet.get(reinsertVessel).route.add(reinsertPos, bestReinsert);
			sol.removedNodes.remove(bestReinsert);
			reinsertTabuList.add(bestReinsert);
			return sol;
		}
		
		/**
		 * Moves a node to its sibling if they are not located on the same vessel.
		 * @param solution the solution to be rearranged
		 * @return the solution with rearrangements if any were done
		 */
		public Solution siblingReunion(Solution solution){
			Solution sol = solutionCopy(solution);
			for (int i = 0; i < sol.fleet.size(); i++) {
				for (int j = 0; j < sol.fleet.get(i).route.size(); j++) {
					Node node = sol.fleet.get(i).route.get(j);
					if(!siblingInVessel(sol.fleet.get(i).route, node) && node.getSibling() != null ){
						if(sol.removedNodes.contains(node.getSibling()) && !reinsertTabuList.contains(node.getSibling())){
							System.out.println("Sibling was in removed nodes, reinserting "+node.getSibling().getNodeNumber()+" "+node.getSibling().getNodeName()+" into vessel "+i);
							sol.fleet.get(i).route.add(j, node.getSibling());
							sol.removedNodes.remove(node.getSibling());
							reinsertTabuList.add(node.getSibling());
							return sol;
						}
						for (int k = 0; k < sol.fleet.size(); k++) {
							if(k == i){
								continue;
							}
							if(siblingInVessel(sol.fleet.get(k).route, node) && !siblingTabuList.contains(node)){
								// Mulighet for forbedring, vi flytter noden til sin sibling
								System.out.println(node.getSibling().getNodeNumber()+" "+node.getSibling().getNodeName()+" was in vessel "+k+", moving node "+node.getNodeNumber()+" "+node.getNodeName()+" from vessel "+i);
								int index = sol.fleet.get(k).route.indexOf(node.getSibling());
								sol.fleet.get(k).route.add(index, node);
								sol.fleet.get(i).route.remove(node);
								siblingTabuList.add(node);
								return sol;
							}
						}
					}
				}
			}
			stopSiblingReunion = true;
			System.out.println("All nodes were with their sibling or has been tried to be moved");
			return sol;
			
		}
		
		/**
		 * Moves a node that is furthest away from all other nodes in its route to another vessel. 
		 * It's sibling is also moved, regardless of where it is located. They are moved to the vessel where they are
		 * closest to all other nodes in that route
		 * @param solution the solution to be rearranged
		 * @return the solution with rearrangements if any were done
		 */
		public Solution siblingPairMover(Solution solution){
			Solution sol = solutionCopy(solution);
			double longestSiblingSailingTime = 0;
			Node furthest = null;
			boolean inRemovedNodes = false;
			for (int i = 0; i < sol.fleet.size(); i++) {
				for (int j = 1; j < sol.fleet.get(i).route.size()-1; j++) {
					if(moveTabuList.contains(sol.fleet.get(i).route.get(j))){
						continue;
					}
					double sailingTime = distanceToAllInRoute(sol.fleet.get(i).route, sol.fleet.get(i).route.get(j));
					if(sailingTime > longestSiblingSailingTime){
						longestSiblingSailingTime = sailingTime;
						furthest = sol.fleet.get(i).route.get(j);
					}
				}
			}
			if(furthest == null){
				for (int i = 0; i < sol.removedNodes.size(); i++) {
					if(moveTabuList.contains(sol.removedNodes.get(i))){
						continue;
					}
					if(sol.removedNodes.contains(sol.removedNodes.get(i).getSibling())){
						furthest = sol.removedNodes.get(i);
						inRemovedNodes = true;
					}
				}
			}
			if(furthest != null){
				if(inRemovedNodes){
					double shortestSailingTime = 10000;
					int vessel = 0;
					for (int i = 0; i < sol.fleet.size(); i++) {
						double sailingTime = distanceToAllInRoute(sol.fleet.get(i).route, furthest);
						if(sailingTime<shortestSailingTime){
							shortestSailingTime = sailingTime;
							vessel = i;
						}
						
					}
					System.out.println("Reinserting them into vessel "+vessel);
					sol.fleet.get(vessel).route.add(1, furthest);
					moveTabuList.add(furthest);
					if(furthest.getSibling() != null){
						sol.fleet.get(vessel).route.add(1, furthest.getSibling());
						moveTabuList.add(furthest.getSibling());
					}
				}else{
					int removeVessel = 0;
					System.out.println("Furthest node is "+furthest.getNodeNumber()+" "+furthest.getNodeName());
					for (int i = 0; i < sol.fleet.size(); i++) {
						if(sol.fleet.get(i).route.contains(furthest)){
							sol.fleet.get(i).route.remove(furthest);
							System.out.println("Removing it from vessel "+i);
							removeVessel = i;
							moveTabuList.add(furthest);
						}
						
					}
					for (int i = 0; i < sol.fleet.size(); i++) {
						if(sol.fleet.get(i).route.contains(furthest.getSibling())){
							sol.fleet.get(i).route.remove(furthest.getSibling());
							System.out.println("Removing its sibling from vessel "+i);
							moveTabuList.add(furthest.getSibling());
						}
					}
					double shortestSailingTime = 10000;
					int vessel = 0;
					for (int i = 0; i < sol.fleet.size(); i++) {
						if(i == removeVessel){
							continue;
						}
						double sailingTime = distanceToAllInRoute(sol.fleet.get(i).route, furthest);
						if(sailingTime<shortestSailingTime){
							shortestSailingTime = sailingTime;
							vessel = i;
						}
						
					}
					System.out.println("Reinserting them into vessel "+vessel);
					sol.fleet.get(vessel).route.add(1, furthest);
					if(furthest.getSibling() != null){
						sol.fleet.get(vessel).route.add(1, furthest.getSibling());
					}
				}
				
			}
			
			return sol;
			
		}
		
		
		/**
		 * Calculates the distance from one node to all other nodes in the same route
		 * @param route the route where the node is located
		 * @param node the node to calculate from
		 * @return the total distance
		 */
		public double distanceToAllInRoute(ArrayList<Node> route, Node node){
			double distance = 0;
			for (int i = 0; i < route.size(); i++) {
				if(route.get(i) == node.getSibling()){
					continue;
				}
				distance += node.getSailingTime(route.get(i));
			}
			return distance;
		}
		
		/**
		 * Checks if a node's sibling is located in the same vessel
		 * @param route the route of the node to be checked
		 * @param node the node to be checked
		 * @return a boolean stating if the sibling is located in the same vessel
		 */
		public boolean siblingInVessel(ArrayList<Node> route, Node node){
			if(route.contains(node.getSibling())){
				return true;
			}
			return false;
		}
		
		/**
		 * Creates a list of all nodes that are not in the removedNodes list, excluding the depots
		 * @param solution the current solution
		 * @return a list of all the nodes that are not removed
		 */
		public ArrayList<Node> allNodesInRoutes(Solution solution){
			ArrayList<Node> allNodes = new ArrayList<Node>();
			for (int i = 0; i < solution.fleet.size(); i++) {
				for (int j = 1; j < solution.fleet.get(i).route.size()-1; j++) {
					Node node = solution.fleet.get(i).route.get(j);
					allNodes.add(node);
					
				}
			}
			return allNodes;
		}
		
		/**
		 * Checks if it is possible to add a node to a vessel without breaking the capacity constraints
		 * @param vessel the vessel the node is to be inserted into
		 * @param node the node
		 * @return a boolean stating if it is possible to insert the node
		 */
		public boolean hasSpace(Vessel vessel, Node node){
			if(node instanceof DeliveryNode){
				if(vessel.voyageCapOKAdd(1, node)){
					return true;
				}
			}
			if(node instanceof PickupNode){
				if(vessel.route.size() == 2){
					return true;
				}
				else if(vessel.voyageCapOKAdd(vessel.route.size()-2, node)){
					return true;
				}
			}
			return false;
		}
		
		/**
		 * Switches routes between vessels to see if delays can be reduced
		 * @param vessels the list of vessels
		 * @return a list of vessels with rearranged routes if any improvements were found
		 */
		public ArrayList<Vessel> vesselSwitch(ArrayList<Vessel> vessels){
			
			ArrayList<Node> tempRoute;
			for (int i = 0; i < vessels.size()-1; i++) {
				ArrayList<Vessel> vesselsCopy = vesselsCloning(vessels);
				tempRoute = (ArrayList<Node>) vessels.get(i).route.clone();
				vesselsCopy.get(i).route = (ArrayList<Node>) vesselsCopy.get(i+1).route.clone();
				vesselsCopy.get(i+1).route = (ArrayList<Node>) tempRoute.clone();
				
				if (TotalCost(vesselsCopy) < TotalCost(vessels) && allVesselsOK(vesselsCopy)) {
					return vesselsCopy;	
				}
			}
			
			return vessels;
		}
		
		/**
		 * Rearranges the order in which siblings are visited, so that delivery nodes are visited before pickup nodes
		 * @param vessels the list of vessels
		 * @return a list of vessels with rearranged routes if any improvements were found
		 */
		public ArrayList<Vessel> siblingOrder(ArrayList<Vessel> vessels){
			boolean changesMade = false;
			ArrayList<Vessel> vesselsCopy = vesselsCloning(vessels);
			for (int i = 0; i < vesselsCopy.size(); i++) {
				for (int j = 1; j < vesselsCopy.get(i).route.size()-1; j++) {
					if (vesselsCopy.get(i).route.get(j+1).getSibling() == vesselsCopy.get(i).route.get(j) && vesselsCopy.get(i).route.get(j) instanceof PickupNode) {
						Node tempNode = vesselsCopy.get(i).route.get(j);
						vesselsCopy.get(i).route.remove(tempNode);
						vesselsCopy.get(i).route.add(j+1, tempNode);
						changesMade = true;
					}
				}
			}
			if (TotalCost(vesselsCopy) <= TotalCost(vessels) && allVesselsOK(vesselsCopy) && changesMade){
				return vesselsCopy;
			}
			return vessels;
		}
		
		
	}

